# PyQt
from PyQt5.QtCore import Qt, QRectF
from PyQt5.QtWidgets import QProgressBar
from PyQt5.QtGui import QPainter, QColor, QPen, QFont, QFontMetricsF
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QProgressBar = createWidgetMenuBase(QProgressBar)

class QtBoxStyleProgressBar7(QProgressBar):
    def __init__(self):
        super(QtBoxStyleProgressBar7, self).__init__(str(Path(__file__)))
        self.setFixedSize(80, 80)
        self.setRange(0, 100)
        self.setValue(80)

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)

        offset = 20
        arc_rect = QRectF(self.rect().left() + offset / 2, self.rect().top() + offset / 2, self.rect().width() - offset, self.rect().height() - offset)

        arc_pen = QPen(Qt.SolidLine)
        arc_pen.setColor(QColor("#7f6ad1"))
        arc_pen.setWidth(18)
        painter.setPen(arc_pen)
        percent = self.value() / self.maximum() if self.maximum() != 0 else 0
        painter.drawArc(arc_rect, 90*16, int(percent*360*16))

        font = QFont()
        font.setPixelSize(15)
        painter.setFont(font)
        font_metrics = QFontMetricsF(font)
        text_width = font_metrics.width(self.text())
        painter.drawText(int(self.width()/2-text_width/2), 45, self.text())
# PyQt

# PySide
# from PySide2.QtCore import Qt, QRectF
# from PySide2.QtWidgets import QProgressBar
# from PySide2.QtGui import QPainter, QColor, QPen, QFont, QFontMetricsF


# class QtBoxStyleProgressBar7(QProgressBar):
#     def __init__(self):
#         super(QtBoxStyleProgressBar7, self).__init__()
#         self.setFixedSize(80, 80)
#         self.setRange(0, 100)
#         self.setValue(80)

#     def paintEvent(self, event):
#         painter = QPainter(self)
#         painter.setRenderHint(QPainter.Antialiasing)

#         offset = 20
#         arc_rect = QRectF(self.rect().left() + offset / 2, self.rect().top() + offset / 2, self.rect().width() - offset, self.rect().height() - offset)

#         arc_pen = QPen(Qt.SolidLine)
#         arc_pen.setColor(QColor("#7f6ad1"))
#         arc_pen.setWidth(18)
#         painter.setPen(arc_pen)
#         percent = self.value() / self.maximum() if self.maximum() != 0 else 0
#         painter.drawArc(arc_rect, 90*16, int(percent*360*16))

#         font = QFont()
#         font.setPixelSize(15)
#         painter.setFont(font)
#         font_metrics = QFontMetricsF(font)
#         text_width = font_metrics.width(self.text())
#         painter.drawText(int(self.width()/2-text_width/2), 45, self.text())
# PySide


# PyQt
from PyQt5.QtWidgets import QLabel
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLabel = createWidgetMenuBase(QLabel)

class QtBoxFuncLabel1(QLabel):
    def __init__(self):
        super(QtBoxFuncLabel1, self).__init__(str(Path(__file__)))
        self.setText("<a href='https://github.com/la-vie-est-belle/qtbox' style='text-decoration:none'>Qt Box link</a>")
        self.setOpenExternalLinks(True)
# PyQt

# PySide
# from PySide2.QtWidgets import QLabel


# class QtBoxFuncLabel1(QLabel):
#     def __init__(self):
#         super(QtBoxFuncLabel1, self).__init__()
#         self.setText("<a href='https://github.com/la-vie-est-belle/qtbox' style='text-decoration:none'>Qt Box link</a>")
#         self.setOpenExternalLinks(True)
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QLabel
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLabel = createWidgetMenuBase(QLabel)

class QtBoxFuncLabel2(QLabel):
    def __init__(self):
        super(QtBoxFuncLabel2, self).__init__(str(Path(__file__)))
        self.setText("I'm editable.")
        self.setTextInteractionFlags(Qt.TextEditorInteraction)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QLabel


# class QtBoxFuncLabel2(QLabel):
#     def __init__(self):
#         super(QtBoxFuncLabel2, self).__init__()
#         self.setText("I'm editable.")
#         self.setTextInteractionFlags(Qt.TextEditorInteraction)
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QLineEdit, QPushButton
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLineEdit = createWidgetMenuBase(QLineEdit)

class QtBoxStyleLineEdit4(QLineEdit):
    def __init__(self):
        super(QtBoxStyleLineEdit4, self).__init__(str(Path(__file__)))
        self.setAttribute(Qt.WA_MacShowFocusRect, False)
        self.setPlaceholderText("search")
        self.setFixedSize(180, 30)
        self.setStyleSheet("""
        QLineEdit {
            border: 1px solid lightgray;
            border-radius: 5px;
            padding-left: 5px;
            padding-right: 60px;
        }
        """)

        self.search_btn = QPushButton(self)
        self.search_btn.setText("Search")
        self.search_btn.setFixedSize(60, self.height())
        self.search_btn.setCursor(Qt.PointingHandCursor)
        self.search_btn.move(self.width()-self.search_btn.width(), 0)
        self.search_btn.setStyleSheet("""
        QPushButton {
            background-color: #2a70f4;
            color: white; 
            font-size: 13px;
            border-top-right-radius: 5px;
            border-bottom-right-radius: 5px;
        }

        QPushButton:hover {
            background-color: #2361d0;
        }
        
        QPushButton:pressed {
            background-color: #2058bc;
        }
        """)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QLineEdit, QPushButton


# class QtBoxStyleLineEdit4(QLineEdit):
#     def __init__(self):
#         super(QtBoxStyleLineEdit4, self).__init__()
#         self.setAttribute(Qt.WA_MacShowFocusRect, False)
#         self.setPlaceholderText("search")
#         self.setFixedSize(180, 30)
#         self.setStyleSheet("""
#         QLineEdit {
#             border: 1px solid lightgray;
#             border-radius: 5px;
#             padding-left: 5px;
#             padding-right: 60px;
#         }
#         """)

#         self.search_btn = QPushButton(self)
#         self.search_btn.setText("Search")
#         self.search_btn.setFixedSize(60, self.height())
#         self.search_btn.setCursor(Qt.PointingHandCursor)
#         self.search_btn.move(self.width()-self.search_btn.width(), 0)
#         self.search_btn.setStyleSheet("""
#         QPushButton {
#             background-color: #2a70f4;
#             color: white;
#             font-size: 13px;
#             border-top-right-radius: 5px;
#             border-bottom-right-radius: 5px;
#         }

#         QPushButton:hover {
#             background-color: #2361d0;
#         }

#         QPushButton:pressed {
#             background-color: #2058bc;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtWidgets import QLabel
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLabel = createWidgetMenuBase(QLabel)
PATH_TO_IMG = str(Path(__file__).parent.parent.parent / "res/images/qt.png").replace("\\", "/")

class QtBoxStyleLabel1(QLabel):
    def __init__(self):
        super(QtBoxStyleLabel1, self).__init__(str(Path(__file__)))
        self.setFixedSize(90, 90)
        self.setStyleSheet("""
        QLabel {
            border-radius: 45px;
            border-image: url(%s)
        }
        """ % (PATH_TO_IMG))
# PyQt

# PySide
# from PySide2.QtWidgets import QLabel


# class QtBoxStyleLabel1(QLabel):
#     def __init__(self):
#         super(QtBoxStyleLabel1, self).__init__()
#         self.setFixedSize(90, 90)
#         self.setStyleSheet("""
#         QLabel {
#             border-radius: 45px;
#             border-image: url(%s)
#         }
#         """ % (PATH_TO_IMG))
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QPainter, QColor, QFont, QFontMetricsF
from PyQt5.QtWidgets import QSlider
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QSlider = createWidgetMenuBase(QSlider)

class QtBoxFuncSlider1(QSlider):
    def __init__(self):
        super(QtBoxFuncSlider1, self).__init__(str(Path(__file__)))
        self.setMinimum(0)
        self.setMaximum(100)
        self.setOrientation(Qt.Horizontal)
        self.is_number_shown = False

    def mousePressEvent(self, event):
        super(QtBoxFuncSlider1, self).mousePressEvent(event)
        if event.button() == Qt.LeftButton:
            self.is_number_shown = True
            self.update()

    def mouseReleaseEvent(self, event):
        super(QtBoxFuncSlider1, self).mouseReleaseEvent(event)
        if event.button() == Qt.LeftButton:
            self.is_number_shown = False
            self.update()

    def paintEvent(self, event):
        super(QtBoxFuncSlider1, self).paintEvent(event)

        if not self.is_number_shown:
            return

        painter = QPainter(self)
        painter.setBrush(QColor(220, 220, 220, 120))

        font = QFont()
        font.setPixelSize(13)
        painter.setFont(font)
        font_metrics = QFontMetricsF(font)
        text_width = font_metrics.width(str(self.value()))
        text_height = font_metrics.height()

        x = (self.value() - self.minimum()) / (self.maximum() - self.minimum()) * (self.width() - text_width)
        painter.setPen(Qt.NoPen)
        painter.drawRoundedRect(x, 0, text_width, text_height*1.3, 3, 3)
        painter.setPen(Qt.SolidLine)
        painter.drawText(int(x), int(text_height), str(self.value()))
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtGui import QPainter, QColor, QFont, QFontMetricsF
# from PySide2.QtWidgets import QSlider


# class QtBoxFuncSlider1(QSlider):
#     def __init__(self):
#         super(QtBoxFuncSlider1, self).__init__(str(Path(__file__)))
#         self.setMinimum(0)
#         self.setMaximum(100)
#         self.setOrientation(Qt.Horizontal)
#         self.is_number_shown = False

#     def mousePressEvent(self, event):
#         super(QtBoxFuncSlider1, self).mousePressEvent(event)
#         if event.button() == Qt.LeftButton:
#             self.is_number_shown = True
#             self.update()

#     def mouseReleaseEvent(self, event):
#         super(QtBoxFuncSlider1, self).mouseReleaseEvent(event)
#         if event.button() == Qt.LeftButton:
#             self.is_number_shown = False
#             self.update()

#     def paintEvent(self, event):
#         super(QtBoxFuncSlider1, self).paintEvent(event)

#         if not self.is_number_shown:
#             return

#         painter = QPainter(self)
#         painter.setBrush(QColor(200, 200, 200, 120))

#         font = QFont()
#         font.setPixelSize(15)
#         painter.setFont(font)
#         font_metrics = QFontMetricsF(font)
#         text_width = font_metrics.width(str(self.value()))
#         text_height = font_metrics.height()

#         x = (self.value() - self.minimum()) / (self.maximum() - self.minimum()) * (self.width() - text_width)
#         painter.setPen(Qt.NoPen)
#         painter.drawRoundedRect(x, 0, text_width, text_height, 2, 2)
#         painter.setPen(Qt.SolidLine)
#         painter.drawText(int(x), int(text_height), str(self.value()))
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QCheckBox
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QCheckBox = createWidgetMenuBase(QCheckBox)

class QtBoxFuncCheckBox2(QCheckBox):
    def __init__(self):
        super(QtBoxFuncCheckBox2, self).__init__(str(Path(__file__)))
        self.setText("Text on the left")
        self.setLayoutDirection(Qt.RightToLeft)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QCheckBox


# class QtBoxFuncCheckBox2(QCheckBox):
#     def __init__(self):
#         super(QtBoxFuncCheckBox2, self).__init__()
#         self.setText("Text on the left")
#         self.setLayoutDirection(Qt.RightToLeft)
# PySide


import sys
import json
from pathlib import Path
from qtbox.utils.check import check_update
from qtbox.utils.output import suppress_stdout_stderr
from qtbox.utils.title import TitleWidget as QSSEditorTitle

from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

from pygments import highlight
from pygments.lexers import CssLexer
from pygments.formatters import HtmlFormatter


RES_PATH = Path(__file__).parent.parent / "res"
with open(RES_PATH / "completion.txt", "r", encoding="utf-8") as f:
    COMPLETER_LIST = [keyword.strip() for keyword in f.readlines()]


class LineNumberArea(QWidget):
    def __init__(self, parent):
        super(LineNumberArea, self).__init__(parent)
        self.parent = parent

    def sizeHint(self):
        return QSize(self.parent.get_line_number_area_width(), 0)

    def paintEvent(self, event):
        self.parent.lineNumberAreaPaintEvent(event)


class CustomTextEdit(QPlainTextEdit):
    qss_signal = pyqtSignal(str)

    def __init__(self):
        super(CustomTextEdit, self).__init__()
        self.copy_btn = QPushButton()
        self.clipboard = QApplication.clipboard()
        self.line_number_area = LineNumberArea(self)
        self.completer = QCompleter(COMPLETER_LIST, self)

        self.redo_stack_list = []
        self.undo_stack_list = []
        self.redo_max_count = 50
        self.undo_max_count = 100
        self.theme = None

        self.set_up()

    def set_up(self):
        self.set_window_attr()
        self.set_object_name()
        self.set_style_sheet()
        self.set_widget()
        self.set_signal()
        self.set_layout()

        self.highlight_line()
        self.update_line_number_area_width()

    def set_window_attr(self):
        self.setMinimumWidth(200)

    def set_object_name(self):
        self.setObjectName("customEdit")
        self.copy_btn.setObjectName("copyBtn")
        self.completer.popup().setObjectName("completer")

    def set_style_sheet(self):
        with open(RES_PATH / "config.json", "r", encoding="utf-8") as f:
            self.theme = json.loads(f.read())["theme"]

        with open(RES_PATH / f"qss/{self.theme}/editor.qss", encoding="utf-8") as f:
            qss_content = f.read()
            self.setStyleSheet(qss_content)
            self.completer.popup().setStyleSheet(qss_content)

    def set_widget(self):
        self.completer.setWidget(self)
        self.setUndoRedoEnabled(False)
        self.setTabStopWidth(self.fontMetrics().width("a")*4)
        self.copy_btn.setToolTip("Click to copy")
        self.copy_btn.setCursor(Qt.PointingHandCursor)
        self.copy_btn.setIcon(QIcon(str(RES_PATH / "images/copy.png")))

    def set_signal(self):
        self.textChanged.connect(self.on_text_changed)
        self.copy_btn.clicked.connect(self.copy)
        self.clipboard.dataChanged.connect(self.on_clipboard_changed)
        self.cursorPositionChanged.connect(self.highlight_line)
        self.updateRequest.connect(self.update_line_number_area)
        self.blockCountChanged.connect(self.update_line_number_area_width)
        self.completer.activated.connect(self.activate_completer)

    def set_layout(self):
        h_layout = QHBoxLayout()
        v_layout = QVBoxLayout(self)
        h_layout.addStretch()
        h_layout.addWidget(self.copy_btn)
        v_layout.addLayout(h_layout)
        v_layout.addStretch()

    def on_text_changed(self):
        self.qss_signal.emit(self.toPlainText())
        self.copy_btn.setToolTip("Click to copy")
        self.copy_btn.setIcon(QIcon(str(RES_PATH / "images/copy.png")))

    def copy(self):
        self.clipboard.setText(self.toPlainText())

    def on_clipboard_changed(self):
        data = self.clipboard.mimeData()
        if data.text() == self.toPlainText():
            self.copy_btn.setIcon(QIcon(str(RES_PATH / "images/check.png")))
            self.copy_btn.setToolTip("Copied")
        else:
            self.copy_btn.setIcon(QIcon(str(RES_PATH / "images/copy.png")))
            self.copy_btn.setToolTip("Click to copy")

    def highlight_line(self):
        extra_selection_list = []
        selection = QTextEdit.ExtraSelection()
        if self.theme == "black":
            selection.format.setBackground(QColor(59, 63, 66, 150))
        else:
            selection.format.setBackground(QColor(242, 242, 242, 150))
        selection.format.setProperty(QTextFormat.FullWidthSelection, True)
        selection.cursor = self.textCursor()
        selection.cursor.clearSelection()
        extra_selection_list.append(selection)
        self.setExtraSelections(extra_selection_list)

    def get_line_number_area_width(self):
        offset = 10
        digit_num = 3
        max_value = max(1, self.blockCount())
        while max_value >= 1000:
            max_value /= 10
            digit_num += 1
        width = offset + self.fontMetrics().width('9') * digit_num
        return width

    def update_line_number_area_width(self):
        self.setViewportMargins(self.get_line_number_area_width()-3, -4, -2, 0)

    def update_line_number_area(self, rect, dy):
        if dy:
            self.line_number_area.scroll(0, dy)
        else:
            self.line_number_area.update(0, rect.y(), self.line_number_area.width(), rect.height())

        if rect.contains(self.viewport().rect()):
            self.update_line_number_area_width()

    def get_current_line_number(self):
        text_cursor = self.textCursor()
        return text_cursor.blockNumber() + 1

    def activate_completer(self, text):
        completer_prefix = self.get_word_under_cursor()
        inserted_text = text.replace(completer_prefix, "")

        text_cursor = self.textCursor()
        text_cursor.insertText(inserted_text)
        current_cursor_position = text_cursor.position()

        self.highlight_text()
        new_cursor_position = text_cursor.position()

        self.move_back_cursor_after_highlight(current_cursor_position, new_cursor_position)

    def get_word_under_cursor(self):
        text_cursor = self.textCursor()
        for _ in range(text_cursor.positionInBlock()):
            text_cursor.movePosition(QTextCursor.Left, QTextCursor.KeepAnchor)
            if " " in text_cursor.selectedText() or ":" in text_cursor.selectedText():
                break
        return text_cursor.selectedText().replace(" ", "").strip()

    def highlight_text(self):
        current_plain_text = self.toPlainText().strip()

        if self.theme == "black":
            formatter = HtmlFormatter(linenos=False, style="paraiso-dark")
            css = "<style>" + formatter.get_style_defs('.highlight').replace("#2f1e2e", "#2b2b2b").replace("#4f424c", "#2b2b2b") + "</style>"
        else:
            formatter = HtmlFormatter(linenos=False, style="tango")
            css = "<style>" + formatter.get_style_defs('.highlight').replace("#ffffcc", "#ffffff").replace("#f8f8f8", "#ffffff") + "</style>"
        html = highlight(current_plain_text, lexer=CssLexer(), formatter=formatter)

        self.document().setHtml(css+html)
        self.moveCursor(QTextCursor.End)
        self.add_undo_command()

    def move_back_cursor_after_highlight(self, pos1, pos2):
        # pos2-1: leave a space
        if pos1 < pos2-1:
            for _ in range(pos2-pos1):
                self.moveCursor(QTextCursor.Left, QTextCursor.MoveAnchor)
        elif pos1 > pos2:
            for _ in range(pos1-pos2):
                self.moveCursor(QTextCursor.Right, QTextCursor.MoveAnchor)

    def redo(self):
        if not self.redo_stack_list:
            return

        if self.document().toHtml() == self.redo_stack_list[-1][0]:
            self.redo_stack_list.pop()
            if not self.redo_stack_list:
                return

        redo_command = self.redo_stack_list.pop()
        self.document().setHtml(redo_command[0])
        original_cursor_position = redo_command[1]
        current_cursor_position = self.textCursor().position()
        self.move_back_cursor_after_highlight(original_cursor_position, current_cursor_position)

    def add_redo_command(self, command):
        if len(self.redo_stack_list) > self.redo_max_count:
            self.redo_stack_list.pop(0)

        self.redo_stack_list.append(command)

    def undo(self):
        if not self.undo_stack_list:
            return

        if self.document().toHtml() == self.undo_stack_list[-1][0]:
            self.redo_stack_list.append(self.undo_stack_list.pop())
            if not self.undo_stack_list:
                return

        undo_command = self.undo_stack_list.pop()
        self.add_redo_command(undo_command)
        self.document().setHtml(undo_command[0])
        original_cursor_position = undo_command[1]
        current_cursor_position = self.textCursor().position()
        self.move_back_cursor_after_highlight(original_cursor_position, current_cursor_position)

    def add_undo_command(self):
        if len(self.undo_stack_list) > self.undo_max_count:
            self.undo_stack_list.pop(0)

        if self.undo_stack_list:
            if self.document().toHtml() != self.undo_stack_list[-1][0]:
                self.undo_stack_list.append((self.document().toHtml(), self.textCursor().position()))
        else:
            self.undo_stack_list.append((self.document().toHtml(), self.textCursor().position()))

    def is_text_cursor_in_brace_block(self):
        """Check if the text cursor is between { and }"""
        left_brace_pos = None
        right_brace_pos = None

        text_cursor = self.textCursor()
        for i in range(text_cursor.position()):
            text_cursor.movePosition(QTextCursor.Left, QTextCursor.KeepAnchor)
            if "{" in text_cursor.selectedText():
                left_brace_pos = i
            elif "}" in text_cursor.selectedText():
                right_brace_pos = i

            if left_brace_pos is not None and right_brace_pos is not None:
                break

        if left_brace_pos is None and right_brace_pos is None:
            return False
        elif left_brace_pos is not None and right_brace_pos is None:
            return True
        elif left_brace_pos is None and right_brace_pos is not None:
            return False

        if left_brace_pos < right_brace_pos:
            return True
        else:
            return False

    def contextMenuEvent(self, event):
        return

    def resizeEvent(self, event):
        cr = self.contentsRect()
        self.line_number_area.setGeometry(QRect(cr.left(), cr.top(), self.get_line_number_area_width(), cr.height()))

    def keyPressEvent(self, event):
        if self.completer.popup().isVisible():
            if event.key() == Qt.Key_Escape or event.key() == Qt.Key_Enter or event.key() == Qt.Key_Return or event.key() == Qt.Key_Tab:
                event.ignore()
                return

        super(CustomTextEdit, self).keyPressEvent(event)
        if event.modifiers() == Qt.ControlModifier:
            if event.key() == Qt.Key_Z:
                self.undo()
            elif event.key() == Qt.Key_V:
                self.highlight_text()
        elif event.modifiers() == Qt.ControlModifier | Qt.ShiftModifier:
            if event.key() == Qt.Key_Z:
                self.redo()
        else:
            if event.key() == Qt.Key_Enter or event.key() == Qt.Key_Return and self.is_text_cursor_in_brace_block():
                self.textCursor().insertText("\t")
            self.add_undo_command()

        word_under_cursor = self.get_word_under_cursor()
        if not word_under_cursor or event.key() == Qt.Key_Left or event.key() == Qt.Key_Right or event.key() == Qt.Key_Up or event.key() == Qt.Key_Down:
            self.completer.popup().setVisible(False)
            return

        if word_under_cursor == "{" and event.key() != Qt.Key_Backspace:
            self.highlight_text()
            return

        if word_under_cursor == "}" and event.key() != Qt.Key_Backspace:
            text_cursor = self.textCursor()
            for _ in range(text_cursor.positionInBlock()):
                text_cursor.movePosition(QTextCursor.Left, QTextCursor.KeepAnchor)
            if text_cursor.selectedText().strip() == "}":
                text_cursor.removeSelectedText()
                text_cursor.insertText("}")
            self.highlight_text()
            return

        self.complete_completer(word_under_cursor)

    def complete_completer(self, word_under_cursor):
        completer_prefix = word_under_cursor
        self.completer.setCompletionPrefix(completer_prefix)

        height = 100
        width = int(self.rect().width() / 2)

        if not self.textCursor().block().text():
            x = 0
        else:
            x = int(self.blockBoundingRect(self.textCursor().block()).width() / len(self.textCursor().block().text()) * self.textCursor().positionInBlock())
            if x >= self.rect().width() - width:
                x = self.rect().width() - width

        y = self.blockBoundingGeometry(self.textCursor().block()).y() - 80

        self.completer.complete(QRect(x, y, width, height))

        # Select the first row by default.
        e = QKeyEvent(QEvent.KeyPress, Qt.Key_Down, Qt.NoModifier)
        QCoreApplication.postEvent(self.completer.popup(), e)

    def lineNumberAreaPaintEvent(self, event):
        painter = QPainter(self.line_number_area)
        if self.theme == "black":
            painter.fillRect(event.rect(), QColor(59, 63, 66, 150))
        else:
            painter.fillRect(event.rect(), QColor(242, 242, 242, 150))

        block = self.firstVisibleBlock()
        block_number = block.blockNumber()
        top = self.blockBoundingGeometry(block).translated(self.contentOffset()).top()
        bottom = top + self.blockBoundingRect(block).height()

        while block.isValid() and top <= event.rect().bottom():
            if block.isVisible() and bottom >= event.rect().top():
                number = block_number + 1
                if number == self.get_current_line_number():
                    painter.setPen(QColor(178, 189, 175, 250))
                else:
                    painter.setPen(QColor(178, 189, 175, 160))

                text_rect = QRectF(0, top-4, self.line_number_area.width(), self.fontMetrics().height())
                painter.drawText(text_rect, Qt.AlignCenter, str(number))

            block = block.next()
            top = bottom
            bottom = top + self.blockBoundingRect(block).height()
            block_number += 1

    def reload_style_sheet(self):
        self.set_style_sheet()
        self.highlight_text()


class DisplayWidget(QWidget):
    def __init__(self):
        super(DisplayWidget, self).__init__()
        self.widget_list = ["QCheckBox", "QComboBox", "QDial", "QLabel", "QLCDNumber", "QLineEdit",
                            "QProgressBar", "QPushButton", "QSlider", "QSpinBox", "QWidget"]
        self.widget_combo_box = QComboBox()
        self.current_widget = None
        self.qss = None

        self.set_up()

    def set_up(self):
        self.set_window_attr()
        self.set_object_name()
        self.set_style_sheet()
        self.set_widget()
        self.set_signal()
        self.set_layout()
        self.change_widget("QCheckBox")

    def set_window_attr(self):
        self.resize(400, 400)
        self.setMinimumWidth(200)
        self.setMinimumHeight(200)

    def set_object_name(self):
        self.setObjectName("displayWidget")
        self.widget_combo_box.setObjectName("widgetComboBox")

    def set_style_sheet(self):
        with open(RES_PATH / "config.json", "r", encoding="utf-8") as f:
            theme = json.loads(f.read())["theme"]

        with open(RES_PATH / f"qss/{theme}/editor.qss", encoding="utf-8") as f:
            self.setStyleSheet(f.read())

        icon_path = str(RES_PATH / "images/up_down.png").replace("\\", "/")
        self.setStyleSheet("""
        QComboBox#widgetComboBox::down-arrow {
            width: 13px;
            height: 13px;
            image: url(%s);
        }
        """ % icon_path)

    def set_widget(self):
        self.widget_combo_box.addItems(self.widget_list)

    def set_signal(self):
        self.widget_combo_box.currentTextChanged.connect(self.change_widget)

    def set_layout(self):
        h_layout = QHBoxLayout()
        v_layout = QVBoxLayout(self)
        h_layout.addStretch()
        h_layout.addWidget(self.widget_combo_box)
        v_layout.addLayout(h_layout)
        v_layout.addStretch()

    def change_widget(self, text):
        if self.current_widget:
            self.current_widget.deleteLater()

        if text == "QCheckBox":
            self.current_widget = QCheckBox(self)
            self.current_widget.setText("Qt Box")
            self.current_widget.setTristate(True)

        elif text == "QComboBox":
            self.current_widget = QComboBox(self)
            self.current_widget.addItems(["1", "2", "3", "4", "5", "6"])

        elif text == "QDial":
            self.current_widget = QDial(self)
            self.current_widget.setMinimum(0)
            self.current_widget.setMaximum(100)
            self.current_widget.setFixedSize(100, 100)

        elif text == "QLabel":
            self.current_widget = QLabel(self)
            self.current_widget.setText("Qt Box")

        elif text == "QLCDNumber":
            self.current_widget = QLCDNumber(self)
            self.current_widget.setDigitCount(6)
            self.current_widget.display(123456)

        elif text == "QLineEdit":
            self.current_widget = QLineEdit(self)
            self.current_widget.setPlaceholderText("Qt Box")

        elif text == "QProgressBar":
            self.current_widget = QProgressBar(self)
            self.current_widget.setMinimum(0)
            self.current_widget.setMaximum(100)
            self.current_widget.setValue(20)

        elif text == "QPushButton":
            self.current_widget = QPushButton(self)
            self.current_widget.setText("BUTTON")

        elif text == "QSlider":
            self.current_widget = QSlider(self)
            self.current_widget.setOrientation(Qt.Horizontal)
            self.current_widget.setMinimum(0)
            self.current_widget.setMaximum(100)

        elif text == "QSpinBox":
            self.current_widget = QSpinBox(self)

        elif text == "QWidget":
            self.current_widget = QWidget(self)
            self.current_widget.setFixedSize(150, 150)
            self.current_widget.setAutoFillBackground(True)

        if self.current_widget:
            self.current_widget.show()
            self.set_current_widget_style_sheet(self.qss)
            self.current_widget.move(int(self.width() / 2 - self.current_widget.width() / 2),
                                     int(self.height() / 2 - self.current_widget.height() / 2))

    def set_current_widget_style_sheet(self, qss):
        with suppress_stdout_stderr():
            self.current_widget.setStyleSheet(qss)
            self.qss = qss

    def resizeEvent(self, event):
        super(DisplayWidget, self).resizeEvent(event)
        if self.current_widget:
            self.current_widget.move(int(self.width()/2-self.current_widget.width()/2), int(self.height()/2-self.current_widget.height()/2))

    def reload_style_sheet(self):
        self.set_style_sheet()


class QSSEditorBody(QSplitter):
    def __init__(self):
        super(QSSEditorBody, self).__init__()
        self.edit = CustomTextEdit()
        self.display_widget = DisplayWidget()

        self.set_up()

    def set_up(self):
        self.set_window_attr()
        self.set_object_name()
        self.set_style_sheet()
        self.set_widget()
        self.set_signal()

    def set_window_attr(self):
        self.resize(800, 600)

    def set_object_name(self):
        self.setObjectName("qssEditorBody")

    def set_style_sheet(self):
        with open(RES_PATH / "config.json", "r", encoding="utf-8") as f:
            theme = json.loads(f.read())["theme"]

        with open(RES_PATH / f"qss/{theme}/editor.qss", encoding="utf-8") as f:
            self.setStyleSheet(f.read())

    def set_widget(self):
        self.addWidget(self.edit)
        self.addWidget(self.display_widget)
        self.setSizes([400, 400])

    def set_signal(self):
        self.edit.qss_signal.connect(self.display_widget.set_current_widget_style_sheet)

    def contextMenuEvent(self, event):
        pass

    def reload_style_sheet(self):
        self.set_style_sheet()
        self.edit.reload_style_sheet()
        self.display_widget.reload_style_sheet()


class QSSEditor(QWidget):
    def __init__(self):
        super(QSSEditor, self).__init__()
        self.qss_editor_title = QSSEditorTitle()
        self.qss_editor_body = QSSEditorBody()

        self.start_x = None
        self.start_y = None

        self.bg_color = None

        self.set_up()

    def set_up(self):
        self.set_window_attr()
        self.set_style_sheet()
        self.set_signal()
        self.set_layout()

    def set_window_attr(self):
        self.resize(850, 650)
        self.qss_editor_title.setMaximumHeight(40)
        self.setWindowFlag(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)

    def set_style_sheet(self):
        with open(RES_PATH / "config.json", "r", encoding="utf-8") as f:
            theme = json.loads(f.read())["theme"]

        if theme == "black":
            self.bg_color = "#3b3f42"
        else:
            self.bg_color = "#f2f2f2"

    def set_signal(self):
        self.qss_editor_title.close_signal.connect(self.close)
        self.qss_editor_title.minimize_signal.connect(self.showMinimized)
        self.qss_editor_title.maximize_signal.connect(self.showMaximized)
        self.qss_editor_title.show_normal_signal.connect(self.showNormal)

    def set_layout(self):
        v_layout = QVBoxLayout(self)
        v_layout.setSpacing(0)
        v_layout.addWidget(self.qss_editor_title)
        v_layout.addWidget(self.qss_editor_body)
        v_layout.setSpacing(5)
        v_layout.setContentsMargins(0, 0, 0, 0)

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.start_x = event.x()
            self.start_y = event.y()

    def mouseMoveEvent(self, event):
        if self.start_x is None or self.start_y is None:
            return

        dis_x = event.x() - self.start_x
        dis_y = event.y() - self.start_y
        self.move(self.x() + dis_x, self.y() + dis_y)

    def mouseReleaseEvent(self, event):
        self.start_x = None
        self.start_y = None

    def paintEvent(self, event):
        super(QSSEditor, self).paintEvent(event)
        painter = QPainter(self)
        painter.setPen(Qt.NoPen)
        painter.setBrush(QColor(self.bg_color))
        painter.setRenderHint(QPainter.Antialiasing)
        painter.drawRoundedRect(self.rect(), 9, 9)

    def showMaximized(self):
        self.move(0, 0)
        desktop = QApplication.desktop()
        self.resize(desktop.width(), desktop.height())

    def showNormal(self):
        super(QSSEditor, self).showNormal()
        self.resize(850, 650)
        desktop = QApplication.desktop()
        self.move(desktop.width() // 2 - self.width() // 2, desktop.height() // 2 - self.height() // 2)

    def reload_style_sheet(self):
        self.set_style_sheet()
        self.qss_editor_title.reload_style_sheet()
        self.qss_editor_body.reload_style_sheet()


def main():
    check_update()
    app = QApplication([])
    qss_editor = QSSEditor()
    qss_editor.show()
    sys.exit(app.exec())


if __name__ == "__main__":
    main()


import json
from pathlib import Path
from qtbox.utils.title import TitleWidget as CodeViewerTitle

from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

from pygments import highlight
from pygments.lexers import PythonLexer
from pygments.formatters import HtmlFormatter


RES_PATH = Path(__file__).parent.parent / "res"


class CodeViewerBody(QTextBrowser):
    def __init__(self):
        super(CodeViewerBody, self).__init__()
        self.clipboard = QApplication.clipboard()
        self.switch_btn = QPushButton()
        self.copy_btn = QPushButton()
        self.pyside_code = ""
        self.pyqt_code = ""
        self.code = ""
        self.theme = None

        self.set_up()

    def set_up(self):
        self.set_object_name()
        self.set_style_sheet()
        self.set_widget()
        self.set_signal()
        self.set_layout()

    def set_object_name(self):
        self.setObjectName("codeViewer")
        self.switch_btn.setObjectName("switchBtn")
        self.copy_btn.setObjectName("copyBtn")

    def set_style_sheet(self):
        with open(RES_PATH / "config.json", "r", encoding="utf-8") as f:
            self.theme = json.loads(f.read())["theme"]

        with open(RES_PATH / f"qss/{self.theme}/viewer.qss", encoding="utf-8") as f:
            self.setStyleSheet(f.read())

    def set_widget(self):
        self.switch_btn.setToolTip("Switch to PySide")
        self.switch_btn.setCursor(Qt.PointingHandCursor)
        self.switch_btn.setIcon(QIcon(str(RES_PATH / "images/switch.png")))
        self.copy_btn.setToolTip("Click to copy")
        self.copy_btn.setCursor(Qt.PointingHandCursor)
        self.copy_btn.setIcon(QIcon(str(RES_PATH / "images/copy.png")))
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOn)

    def set_signal(self):
        self.copy_btn.clicked.connect(self.copy)
        self.switch_btn.clicked.connect(self.switch_code)
        self.clipboard.dataChanged.connect(self.on_clipboard_changed)

    def set_layout(self):
        h_layout = QHBoxLayout()
        v_layout = QVBoxLayout(self)
        h_layout.addStretch()
        h_layout.addWidget(self.switch_btn)
        h_layout.addWidget(self.copy_btn)
        v_layout.addLayout(h_layout)
        v_layout.addStretch()

    def set_code(self, code):
        self.code = code
        if self.theme == "black":
            formatter = HtmlFormatter(linenos=True, style="paraiso-dark")
            css = "<style>" + formatter.get_style_defs('.highlight').replace("#2f1e2e", "#2b2b2b").replace("#4f424c", "#2b2b2b") + "</style>"
        else:
            formatter = HtmlFormatter(linenos=True, style="tango")
            css = "<style>" + formatter.get_style_defs('.highlight').replace("#ffffcc", "#ffffff").replace("#f8f8f8", "#ffffff") + "</style>"
        html = highlight(self.code, lexer=PythonLexer(), formatter=formatter)
        self.setHtml(css+html)
        self.horizontalScrollBar().move(0, 0)

        self.copy_btn.setIcon(QIcon(str(RES_PATH / "images/copy.png")))
        self.copy_btn.setToolTip("Click to copy")

    def copy(self):
        self.clipboard.setText(self.code)

    def on_clipboard_changed(self):
        data = self.clipboard.mimeData()
        if data.text() == self.code:
            self.copy_btn.setIcon(QIcon(str(RES_PATH / "images/check.png")))
            self.copy_btn.setToolTip("Copied")
        else:
            self.copy_btn.setIcon(QIcon(str(RES_PATH / "images/copy.png")))
            self.copy_btn.setToolTip("Click to copy")

    def contextMenuEvent(self, event):
        pass

    def switch_code(self):
        if self.switch_btn.toolTip() == "Switch to PySide":
            self.switch_btn.setToolTip("Switch to PyQt")
            self.set_code(self.pyside_code)
        else:
            self.switch_btn.setToolTip("Switch to PySide")
            self.set_code(self.pyqt_code)

    def reload_style_sheet(self):
        self.set_style_sheet()


class CodeViewer(QWidget):
    def __init__(self):
        super(CodeViewer, self).__init__()
        self.code_viewer_title = CodeViewerTitle()
        self.code_viewer_body = CodeViewerBody()

        self.start_x = None
        self.start_y = None

        self.bg_color = None

        self.set_up()

    def set_up(self):
        self.set_window_attr()
        self.set_style_sheet()
        self.set_signal()
        self.set_layout()

    def set_window_attr(self):
        self.resize(700, 500)
        self.setWindowFlag(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)

    def set_style_sheet(self):
        with open(RES_PATH / "config.json", "r", encoding="utf-8") as f:
            theme = json.loads(f.read())["theme"]

        if theme == "black":
            self.bg_color = "#3b3f42"
        else:
            self.bg_color = "#f2f2f2"

    def set_signal(self):
        self.code_viewer_title.close_signal.connect(self.close)
        self.code_viewer_title.minimize_signal.connect(self.showMinimized)
        self.code_viewer_title.maximize_signal.connect(self.showMaximized)
        self.code_viewer_title.show_normal_signal.connect(self.showNormal)

    def set_layout(self):
        v_layout = QVBoxLayout(self)
        v_layout.setSpacing(0)
        v_layout.addWidget(self.code_viewer_title)
        v_layout.addWidget(self.code_viewer_body)
        v_layout.setSpacing(5)
        v_layout.setContentsMargins(0, 0, 0, 0)

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.start_x = event.x()
            self.start_y = event.y()

    def mouseMoveEvent(self, event):
        if self.start_x is None or self.start_y is None:
            return

        dis_x = event.x() - self.start_x
        dis_y = event.y() - self.start_y
        self.move(self.x() + dis_x, self.y() + dis_y)

    def mouseReleaseEvent(self, event):
        self.start_x = None
        self.start_y = None

    def paintEvent(self, event):
        super(CodeViewer, self).paintEvent(event)
        painter = QPainter(self)
        painter.setPen(Qt.NoPen)
        painter.setBrush(QColor(self.bg_color))
        painter.setRenderHint(QPainter.Antialiasing)
        painter.drawRoundedRect(self.rect(), 9, 9)

    def showMaximized(self):
        self.move(0, 0)
        desktop = QApplication.desktop()
        self.resize(desktop.width(), desktop.height())

    def showNormal(self):
        super(CodeViewer, self).showNormal()
        self.resize(700, 500)
        desktop = QApplication.desktop()
        self.move(desktop.width() // 2 - self.width() // 2, desktop.height() // 2 - self.height() // 2)

    def set_pyqt_and_pyside_property(self, pyqt_code, pyside_code):
        self.code_viewer_body.pyqt_code = pyqt_code
        self.code_viewer_body.pyside_code = pyside_code

    def set_code(self, code):
        self.code_viewer_body.set_code(code)

    def reload_style_sheet(self):
        self.set_style_sheet()
        self.code_viewer_title.reload_style_sheet()
        self.code_viewer_body.reload_style_sheet()
        self.code_viewer_body.set_code(self.code_viewer_body.code)



# PyQt
from PyQt5.QtCore import QTimer
from PyQt5.QtWidgets import QLCDNumber
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLCDNumber = createWidgetMenuBase(QLCDNumber)

class QtBoxFuncLCDNumber2(QLCDNumber):
    def __init__(self):
        super(QtBoxFuncLCDNumber2, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 50)
        self.text = "HELLO"
        self.setDigitCount(len(self.text))
        self.display("")

        self.count = 0
        self.timer = QTimer()
        self.timer.timeout.connect(self.update_display)
        self.timer.start(1000)

    def update_display(self):
        self.count += 1
        if self.count > len(self.text):
            self.display(self.text+(self.count-len(self.text))*" ")
        else:
            self.display(self.text[0:self.count])

        if self.count == 2*len(self.text):
            self.count = 0
# PyQt

# PySide
# from PySide2.QtCore import QTimer
# from PySide2.QtWidgets import QLCDNumber


# class QtBoxFuncLCDNumber2(QLCDNumber):
#     def __init__(self):
#         super(QtBoxFuncLCDNumber2, self).__init__()
#         self.setFixedSize(150, 50)
#         self.text = "HELLO"
#         self.setDigitCount(len(self.text))
#         self.display("")

#         self.count = 0
#         self.timer = QTimer()
#         self.timer.timeout.connect(self.update_display)
#         self.timer.start(1000)

#     def update_display(self):
#         self.count += 1
#         if self.count > len(self.text):
#             self.display(self.text+(self.count-len(self.text))*" ")
#         else:
#             self.display(self.text[0:self.count])

#         if self.count == 2*len(self.text):
#             self.count = 0
# PySide


# PyQt
from PyQt5.QtWidgets import QCheckBox
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QCheckBox = createWidgetMenuBase(QCheckBox)
PATH_TO_IMG1 = str(Path(__file__).parent.parent.parent / "res/images/close.png").replace("\\", "/")
PATH_TO_IMG2 = str(Path(__file__).parent.parent.parent / "res/images/minimize.png").replace("\\", "/")
PATH_TO_IMG3 = str(Path(__file__).parent.parent.parent / "res/images/check.png").replace("\\", "/")

class QtBoxStyleCheckBox1(QCheckBox):
    def __init__(self):
        super(QtBoxStyleCheckBox1, self).__init__(str(Path(__file__)))
        self.setText("Qt Box")
        self.setTristate(True)
        self.setStyleSheet("""
        QCheckBox {
            spacing: 5px;
            font-size: 15px;
            font-weight: bold;
        }
        
        QCheckBox::indicator {
            width: 16px;
            height: 16px;
            border: 2px solid black;
            border-radius: 6px;
        }
        
        QCheckBox::indicator:unchecked {
            image: url(%s);
            background-color: black;
        }
        
        QCheckBox::indicator:indeterminate {
            image: url(%s);
            background-color: black;
        }
        
        QCheckBox::indicator:checked {
            image: url(%s);
            background-color: black;
        }
        """ % (PATH_TO_IMG1, PATH_TO_IMG2, PATH_TO_IMG3))
# PyQt

# PySide
# from PySide2.QtWidgets import QCheckBox


# class QtBoxStyleCheckBox1(QCheckBox):
#     def __init__(self):
#         super(QtBoxStyleCheckBox1, self).__init__()
#         self.setText("Qt Box")
#         self.setTristate(True)
#         self.setStyleSheet("""
#         QCheckBox {
#             spacing: 5px;
#             font-size: 15px;
#             font-weight: bold;
#         }

#         QCheckBox::indicator {
#             width: 16px;
#             height: 16px;
#             border: 2px solid black;
#             border-radius: 6px;
#         }

#         QCheckBox::indicator:unchecked {
#             image: url(%s);
#             background-color: black;
#         }

#         QCheckBox::indicator:indeterminate {
#             image: url(%s);
#             background-color: black;
#         }

#         QCheckBox::indicator:checked {
#             image: url(%s);
#             background-color: black;
#         }
#         """ % (PATH_TO_IMG1, PATH_TO_IMG2, PATH_TO_IMG3))
# PySide


# PyQt
from PyQt5.QtCore import Qt, QRectF
from PyQt5.QtWidgets import QPushButton
from PyQt5.QtGui import QPainter, QPen, QColor, QLinearGradient, QFont, QFontMetricsF
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QPushButton = createWidgetMenuBase(QPushButton)

class QtBoxStyleButton10(QPushButton):
    def __init__(self):
        super(QtBoxStyleButton10, self).__init__(str(Path(__file__)))
        self.btn_rect = None
        self.btn_state = "off"
        self.setFixedSize(90, 90)

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)

        if self.btn_state == "on":
            offset = 40
            font_size = 15
        else:
            offset = 30
            font_size = 20

        painter.setPen(Qt.NoPen)
        base_linear_gradient = QLinearGradient(self.rect().topLeft(), self.rect().bottomRight())
        base_linear_gradient.setColorAt(0, QColor("#757575"))
        base_linear_gradient.setColorAt(1, QColor("#616161"))
        painter.setBrush(base_linear_gradient)
        painter.drawEllipse(self.rect())

        self.btn_rect = QRectF(self.rect().left()+offset/2, self.rect().top()+offset/2, self.rect().width()-offset, self.rect().height()-offset)
        btn_linear_gradient = QLinearGradient(self.btn_rect.topLeft(), self.btn_rect.bottomRight())
        btn_linear_gradient.setColorAt(0, QColor("#f17075"))
        btn_linear_gradient.setColorAt(1, QColor("#df5459"))
        painter.setBrush(btn_linear_gradient)
        painter.drawEllipse(self.btn_rect)

        pen = QPen(Qt.SolidLine)
        pen.setColor(Qt.white)
        painter.setPen(pen)

        font = QFont()
        font.setBold(True)
        font.setPixelSize(font_size)
        painter.setFont(font)
        font_metrics = QFontMetricsF(font)
        text_width = font_metrics.width("BTN")
        painter.drawText(int(self.rect().width()/2-text_width/2), int(self.rect().height()/2+5), "BTN")

    def mousePressEvent(self, event):
        if event.button() != Qt.LeftButton or not self.btn_rect.contains(event.pos()):
            return

        self.btn_state = "on"
        self.update()

    def mouseReleaseEvent(self, event):
        self.btn_state = "off"
        self.update()
# PyQt

# PySide
# from PySide2.QtCore import Qt, QRectF
# from PySide2.QtWidgets import QPushButton
# from PySide2.QtGui import QPainter, QPen, QColor, QLinearGradient, QFont, QFontMetricsF


# class QtBoxStyleButton10(QPushButton):
#     def __init__(self):
#         super(QtBoxStyleButton10, self).__init__()
#         self.btn_rect = None
#         self.btn_state = "off"
#         self.setFixedSize(90, 90)

#     def paintEvent(self, event):
#         painter = QPainter(self)
#         painter.setRenderHint(QPainter.Antialiasing)

#         if self.btn_state == "on":
#             offset = 40
#             font_size = 15
#         else:
#             offset = 30
#             font_size = 20

#         painter.setPen(Qt.NoPen)
#         base_linear_gradient = QLinearGradient(self.rect().topLeft(), self.rect().bottomRight())
#         base_linear_gradient.setColorAt(0, QColor("#757575"))
#         base_linear_gradient.setColorAt(1, QColor("#616161"))
#         painter.setBrush(base_linear_gradient)
#         painter.drawEllipse(self.rect())

#         self.btn_rect = QRectF(self.rect().left()+offset/2, self.rect().top()+offset/2, self.rect().width()-offset, self.rect().height()-offset)
#         btn_linear_gradient = QLinearGradient(self.btn_rect.topLeft(), self.btn_rect.bottomRight())
#         btn_linear_gradient.setColorAt(0, QColor("#f17075"))
#         btn_linear_gradient.setColorAt(1, QColor("#df5459"))
#         painter.setBrush(btn_linear_gradient)
#         painter.drawEllipse(self.btn_rect)

#         pen = QPen(Qt.SolidLine)
#         pen.setColor(Qt.white)
#         painter.setPen(pen)

#         font = QFont()
#         font.setBold(True)
#         font.setPixelSize(font_size)
#         painter.setFont(font)
#         font_metrics = QFontMetricsF(font)
#         text_width = font_metrics.width("BTN")
#         painter.drawText(int(self.rect().width()/2-text_width/2), int(self.rect().height()/2+5), "BTN")

#     def mousePressEvent(self, event):
#         if event.button() != Qt.LeftButton or not self.btn_rect.contains(event.pos()):
#             return

#         self.btn_state = "on"
#         self.update()

#     def mouseReleaseEvent(self, event):
#         self.btn_state = "off"
#         self.update()
# PySide


import json
from pathlib import Path
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

RES_PATH = Path(__file__).parent.parent / "res"


class ContextMenu(QMenu):
    view_code_signal = pyqtSignal()
    download_code_signal = pyqtSignal()

    def __init__(self):
        super(ContextMenu, self).__init__()
        self.view_code_action = QAction(QIcon(str(RES_PATH / "images/open_eye.png")), "View Code", self)
        self.download_code_action = QAction(QIcon(str(RES_PATH / "images/download.png")), "Download Code", self)

        self.set_up()

    def set_up(self):
        self.set_action()
        self.set_signal()
        self.set_style_sheet()

    def set_action(self):
        self.addAction(self.view_code_action)
        self.addAction(self.download_code_action)

    def set_signal(self):
        self.view_code_action.triggered.connect(self.view_code_signal.emit)
        self.download_code_action.triggered.connect(self.download_code_signal.emit)

    def set_style_sheet(self):
        with open(RES_PATH / "config.json", "r", encoding="utf-8") as f:
            theme = json.loads(f.read())["theme"]

        with open(RES_PATH / f"qss/{theme}/menu.qss", encoding="utf-8") as f:
            self.setStyleSheet(f.read())

    def reload_style_sheet(self):
        self.set_style_sheet()


def createWidgetMenuBase(cls):
    class WidgetMenuBase(cls):
        view_code_signal = pyqtSignal(str)
        download_code_signal = pyqtSignal(str)

        def __init__(self, code_file_path):
            super(WidgetMenuBase, self).__init__()
            self.code_file_path = code_file_path
            self.context_menu = ContextMenu()

        def contextMenuEvent(self, event):
            self.context_menu.view_code_signal.connect(lambda: self.view_code_signal.emit(self.code_file_path))
            self.context_menu.download_code_signal.connect(lambda: self.download_code_signal.emit(self.code_file_path))
            self.context_menu.exec(event.globalPos())

    return WidgetMenuBase



# PyQt
from PyQt5.QtCore import QTimer
from PyQt5.QtWidgets import QLabel
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLabel = createWidgetMenuBase(QLabel)

class QtBoxFuncLabel3(QLabel):
    def __init__(self):
        super(QtBoxFuncLabel3, self).__init__(str(Path(__file__)))
        self.text = ""
        self.count = 0
        self.timer = QTimer()
        self.timer.timeout.connect(self.update_text)
        self.type_text("Typewriter effect")

    def type_text(self, text):
        self.text = text
        self.count = 0
        self.timer.stop()
        self.timer.start(500)

    def update_text(self):
        self.count += 1
        self.setText(self.text[0:self.count])

        if self.count >= len(self.text):
            self.count = 0
            self.timer.stop()
# PyQt

# PySide
# from PySide2.QtCore import QTimer
# from PySide2.QtWidgets import QLabel


# class QtBoxFuncLabel3(QLabel):
#     def __init__(self):
#         super(QtBoxFuncLabel3, self).__init__()
#         self.text = ""
#         self.count = 0
#         self.timer = QTimer()
#         self.timer.timeout.connect(self.update_text)
#         self.type_text("Typewriter effect")

#     def type_text(self, text):
#         self.text = text
#         self.count = 0
#         self.timer.stop()
#         self.timer.start(500)

#     def update_text(self):
#         self.count += 1
#         self.setText(self.text[0:self.count])
#
#         if self.count >= len(self.text):
#             self.count = 0
#             self.timer.stop()
# PySide


# PyQt
from PyQt5.QtCore import QThread, pyqtSignal
from PyQt5.QtWidgets import QPushButton, QMessageBox
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QPushButton = createWidgetMenuBase(QPushButton)

class QtBoxFuncButton2(QPushButton):
    def __init__(self):
        super(QtBoxFuncButton2, self).__init__(str(Path(__file__)))
        self.setText("Login")
        self.clicked.connect(self.start_verify_thread)

        self.verify_thread = VerifyThread()
        self.verify_thread.verify_signal.connect(self.do_after_verification)

    def start_verify_thread(self):
        self.verify_thread.start()

    def do_after_verification(self, verify_result):
        if not verify_result:
            self.verify_not_ok()
        else:
            self.verify_ok()

    def verify_not_ok(self):
        QMessageBox.critical(self, "Verify", "Wrong username or password")

    def verify_ok(self):
        QMessageBox.information(self, "Verify", "Login successfully")


class VerifyThread(QThread):
    verify_signal = pyqtSignal(bool)

    def __init__(self):
        super(VerifyThread, self).__init__()

    def run(self):
        self.verify_signal.emit(self.verify())

    def verify(self):
        """
        verify username and password here
        :return: True or False
        """
        ...
        return False
# PyQt


# PySide
# from PySide2.QtCore import QThread, Signal
# from PySide2.QtWidgets import QPushButton, QMessageBox


# class QtBoxFuncButton2(QPushButton):
#     def __init__(self):
#         super(QtBoxFuncButton2, self).__init__()
#         self.setText("Login")
#         self.clicked.connect(self.start_verify_thread)

#         self.verify_thread = VerifyThread()
#         self.verify_thread.verify_signal.connect(self.do_after_verification)

#     def start_verify_thread(self):
#         self.verify_thread.start()

#     def do_after_verification(self, verify_result):
#         if not verify_result:
#             self.verify_not_ok()
#         else:
#             self.verify_ok()

#     def verify_not_ok(self):
#         QMessageBox.critical(self, "Verify", "Wrong username or password")

#     def verify_ok(self):
#         QMessageBox.information(self, "Verify", "Login successfully")


# class VerifyThread(QThread):
#     verify_signal = Signal(bool)

#     def __init__(self):
#         super(VerifyThread, self).__init__()

#     def run(self):
#         self.verify_signal.emit(self.verify())

#     def verify(self):
#         """
#         verify username and password here
#         :return: True or False
#         """
#         ...
#         return False
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QComboBox, QCompleter
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QComboBox = createWidgetMenuBase(QComboBox)

class QtBoxFuncComboBox2(QComboBox):
    def __init__(self):
        super(QtBoxFuncComboBox2, self).__init__(str(Path(__file__)))
        self.setEditable(True)
        self.addItems(['1', '2', '3', '4', '5', '6'])
        comp = QCompleter(["1a", "1b", "1c"])
        comp.setFilterMode(Qt.MatchContains)
        self.setCompleter(comp)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QComboBox, QCompleter


# class QtBoxFuncComboBox2(QComboBox):
#     def __init__(self):
#         super(QtBoxFuncComboBox2, self).__init__()
#         self.setEditable(True)
#         self.addItems(['1', '2', '3', '4', '5', '6'])
#         comp = QCompleter(["1a", "1b", "1c"])
#         comp.setFilterMode(Qt.MatchContains)
#         self.setCompleter(comp)
# PySide


# PyQt
from PyQt5.QtWidgets import QLabel, QGraphicsBlurEffect
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLabel = createWidgetMenuBase(QLabel)

class QtBoxStyleLabel3(QLabel):
    def __init__(self):
        super(QtBoxStyleLabel3, self).__init__(str(Path(__file__)))
        self.setText("Blur")
        self.blur()

    def blur(self):
        blur = QGraphicsBlurEffect(self)
        blur.setBlurRadius(6)
        blur.setBlurHints(QGraphicsBlurEffect.QualityHint)
        self.setGraphicsEffect(blur)
# PyQt

# PySide
# from PySide2.QtWidgets import QLabel, QGraphicsBlurEffect


# class QtBoxStyleLabel3(QLabel):
#     def __init__(self):
#         super(QtBoxStyleLabel3, self).__init__()
#         self.setText("Blur")
#         self.blur()

#     def blur(self):
#         blur = QGraphicsBlurEffect(self)
#         blur.setBlurRadius(6)
#         blur.setBlurHints(QGraphicsBlurEffect.QualityHint)
#         self.setGraphicsEffect(blur)
# PySide


# PyQt
from PyQt5.QtCore import QTimer
from PyQt5.QtWidgets import QLabel
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLabel = createWidgetMenuBase(QLabel)

class QtBoxStyleLabel4(QLabel):
    def __init__(self):
        super(QtBoxStyleLabel4, self).__init__(str(Path(__file__)))
        self.setText("Change text color")
        self.color_list = ["rgb(255, 0, 0)", "rgb(255, 165, 0)", "rgb(255, 255, 0)",
                           "rgb(0, 255, 0)", "rgb(0, 127, 255)", "rgb(0, 0, 255)",
                           "rgb(139, 0, 255)"]

        self.count = 0
        self.timer = QTimer()
        self.timer.timeout.connect(self.change_color)
        self.timer.start(100)

    def change_color(self):
        color = self.color_list[self.count]
        self.setStyleSheet(f"""
        color: {color}
        """)

        self.count += 1
        if self.count >= len(self.color_list):
            self.count = 0
# PyQt

# PySide
# from PySide2.QtCore import QTimer
# from PySide2.QtWidgets import QLabel


# class QtBoxStyleLabel4(QLabel):
#     def __init__(self):
#         super(QtBoxStyleLabel4, self).__init__()
#         self.setText("Change text color")
#         self.color_list = ["rgb(255, 0, 0)", "rgb(255, 165, 0)", "rgb(255, 255, 0)",
#                            "rgb(0, 255, 0)", "rgb(0, 127, 255)", "rgb(0, 0, 255)",
#                            "rgb(139, 0, 255)"]

#         self.count = 0
#         self.timer = QTimer()
#         self.timer.timeout.connect(self.change_color)
#         self.timer.start(100)

#     def change_color(self):
#         color = self.color_list[self.count]
#         self.setStyleSheet(f"""
#         color: {color}
#         """)

#         self.count += 1
#         if self.count >= len(self.color_list):
#             self.count = 0
# PySide


# PyQt
from PyQt5.QtWidgets import QPushButton
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QPushButton = createWidgetMenuBase(QPushButton)

class QtBoxStyleButton3(QPushButton):
    def __init__(self):
        super(QtBoxStyleButton3, self).__init__(str(Path(__file__)))
        self.setFixedSize(90, 90)
        self.setText("BTN")
        self.setStyleSheet("""
        QPushButton {
            background-color: qlineargradient(x1:1, y1:0, x2:1, y2:1, stop:0 #454b4f, stop: 1 #1d2225);
            color: white;
            border-radius: 45px;
            font-size: 18px;
            font-weight: bold;
            border: 10px solid #f9f9f9;
        }
        
        QPushButton:hover {
            border: 10px solid white;
            background-color: qlineargradient(x1:1, y1:0, x2:1, y2:1, stop:0 #35393c, stop: 1 #101214);
        }
        
        QPushButton:pressed {
            color: black;
            border: 10px solid black;
            background-color: white;
        }
        """)
# PyQt

# PySide
# from PySide2.QtWidgets import QPushButton


# class QtBoxStyleButton3(QPushButton):
#     def __init__(self):
#         super(QtBoxStyleButton3, self).__init__()
#         self.setFixedSize(90, 90)
#         self.setText("BTN")
#         self.setStyleSheet("""
#         QPushButton {
#             background-color: qlineargradient(x1:1, y1:0, x2:1, y2:1, stop:0 #454b4f, stop: 1 #1d2225);
#             color: white;
#             border-radius: 45px;
#             font-size: 18px;
#             font-weight: bold;
#             border: 10px solid #f9f9f9;
#         }

#         QPushButton:hover {
#             border: 10px solid white;
#             background-color: qlineargradient(x1:1, y1:0, x2:1, y2:1, stop:0 #35393c, stop: 1 #101214);
#         }

#         QPushButton:pressed {
#             color: black;
#             border: 10px solid black;
#             background-color: white;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtGui import QIcon
from PyQt5.QtCore import QSize
from PyQt5.QtWidgets import QPushButton
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QPushButton = createWidgetMenuBase(QPushButton)
PATH_TO_IMG = str(Path(__file__).parent.parent.parent / "res/images/plus.png").replace("\\", "/")

class QtBoxStyleButton5(QPushButton):
    def __init__(self):
        super(QtBoxStyleButton5, self).__init__(str(Path(__file__)))
        self.setFixedSize(90, 90)
        self.setIconSize(QSize(80, 80))
        self.setIcon(QIcon(PATH_TO_IMG))
        self.setStyleSheet("""
        QPushButton {
            background-color: qlineargradient(x1:1, y1:0, x2:0, y2:1, stop:0 #f9ab6a, stop: 1 #f08833);
            border-radius: 45px;
            border: none
        }

        QPushButton:hover {
            background-color: qlineargradient(x1:1, y1:0, x2:0, y2:1, stop:0 #f7a25b, stop: 1 #f19040);
        }

        QPushButton:pressed {
            background-color: qlineargradient(x1:1, y1:0, x2:0, y2:1, stop:0 #e39b5f, stop: 1 #e58637);
        }
        """)
# PyQt

# PySide
# from PySide2.QtGui import QIcon
# from PySide2.QtCore import QSize
# from PySide2.QtWidgets import QPushButton


# class QtBoxStyleButton5(QPushButton):
#     def __init__(self):
#         super(QtBoxStyleButton5, self).__init__()
#         self.setFixedSize(90, 90)
#         self.setIconSize(QSize(80, 80))
#         self.setIcon(QIcon(PATH_TO_IMG))
#         self.setStyleSheet("""
#         QPushButton {
#             background-color: qlineargradient(x1:1, y1:0, x2:0, y2:1, stop:0 #f9ab6a, stop: 1 #f08833);
#             border-radius: 45px;
#             border: none
#         }

#         QPushButton:hover {
#             background-color: qlineargradient(x1:1, y1:0, x2:0, y2:1, stop:0 #f7a25b, stop: 1 #f19040);
#         }

#         QPushButton:pressed {
#             background-color: qlineargradient(x1:1, y1:0, x2:0, y2:1, stop:0 #e39b5f, stop: 1 #e58637);
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtWidgets import QSpinBox
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QSpinBox = createWidgetMenuBase(QSpinBox)

class QtBoxFuncSpinBox1(QSpinBox):
    def __init__(self):
        super(QtBoxFuncSpinBox1, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 30)
        self.setPrefix("Sth: ")
        self.setSuffix(" PCS")
# PyQt

# PySide
# from PySide2.QtWidgets import QSpinBox


# class QtBoxFuncSpinBox1(QSpinBox):
#     def __init__(self):
#         super(QtBoxFuncSpinBox1, self).__init__(str(Path(__file__)))
#         self.setFixedSize(150, 30)
#         self.setPrefix("Sth: ")
#         self.setSuffix(" PCS")
# PySide


# PyQt
from PyQt5.QtWidgets import QLCDNumber
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLCDNumber = createWidgetMenuBase(QLCDNumber)

class QtBoxStyleLCDNumber3(QLCDNumber):
    def __init__(self):
        super(QtBoxStyleLCDNumber3, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 50)
        self.setDigitCount(6)
        self.display(123456)
        self.setSegmentStyle(QLCDNumber.Flat)
        self.setStyleSheet("""
        QLCDNumber {
            background-color: #98a7a7;
            border: none;
            color: black;
        }
        """)
# PyQt

# PySide
# from PySide2.QtWidgets import QLCDNumber


# class QtBoxStyleLCDNumber3(QLCDNumber):
#     def __init__(self):
#         super(QtBoxStyleLCDNumber3, self).__init__()
#         self.setFixedSize(150, 50)
#         self.setDigitCount(6)
#         self.display(123456)
#         self.setSegmentStyle(QLCDNumber.Flat)
#         self.setStyleSheet("""
#         QLCDNumber {
#             background-color: #98a7a7;
#             border: none;
#             color: black;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtGui import QColor
from PyQt5.QtWidgets import QProgressBar, QGraphicsDropShadowEffect
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QProgressBar = createWidgetMenuBase(QProgressBar)

class QtBoxStyleProgressBar4(QProgressBar):
    def __init__(self):
        super(QtBoxStyleProgressBar4, self).__init__(str(Path(__file__)))
        self.setTextVisible(False)
        self.setMaximumHeight(10)
        self.setRange(0, 100)
        self.setValue(80)
        self.setStyleSheet("""
        QProgressBar {
            border: 1px solid lightgray;
            background-color: #2b241c;
            border-radius: 5px;
        }

        QProgressBar::chunk {
            background-color: qlineargradient(x1:0, y1:0, x2:1, y2:1, stop:0 #f9dc98, stop:1 #f4b543);
            border-radius: 4px;
        }
        """)

        self.add_shadow()

    def add_shadow(self):
        shadow = QGraphicsDropShadowEffect(self)
        shadow.setBlurRadius(40)
        shadow.setOffset(0, 0)
        shadow.setColor(QColor("#f1b143"))
        self.setGraphicsEffect(shadow)
# PyQt

# PySide
# from PySide2.QtGui import QColor
# from PySide2.QtWidgets import QProgressBar, QGraphicsDropShadowEffect


# class QtBoxStyleProgressBar4(QProgressBar):
#     def __init__(self):
#         super(QtBoxStyleProgressBar4, self).__init__()
#         self.setTextVisible(False)
#         self.setMaximumHeight(10)
#         self.setRange(0, 100)
#         self.setValue(80)
#         self.setStyleSheet("""
#         QProgressBar {
#             border: 1px solid lightgray;
#             background-color: #2b241c;
#             border-radius: 5px;
#         }

#         QProgressBar::chunk {
#             background-color: qlineargradient(x1:0, y1:0, x2:1, y2:1, stop:0 #f9dc98, stop:1 #f4b543);
#             border-radius: 4px;
#         }
#         """)

#         self.add_shadow()

#     def add_shadow(self):
#         shadow = QGraphicsDropShadowEffect(self)
#         shadow.setBlurRadius(40)
#         shadow.setOffset(0, 0)
#         shadow.setColor(QColor("#f1b143"))
#         self.setGraphicsEffect(shadow)
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QPainter
from PyQt5.QtWidgets import QWidget
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QWidget = createWidgetMenuBase(QWidget)

class QtBoxStyleWidget1(QWidget):
    def __init__(self):
        super(QtBoxStyleWidget1, self).__init__(str(Path(__file__)))
        self.setFixedSize(100, 100)
        self.setWindowFlag(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setPen(Qt.NoPen)
        painter.setBrush(Qt.lightGray)
        painter.drawRoundedRect(self.rect(), 10, 10)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtGui import QPainter
# from PySide2.QtWidgets import QWidget


# class QtBoxStyleWidget1(QWidget):
#     def __init__(self):
#         super(QtBoxStyleWidget1, self).__init__()
#         self.setFixedSize(100, 100)
#         self.setWindowFlag(Qt.FramelessWindowHint)
#         self.setAttribute(Qt.WA_TranslucentBackground)

#     def paintEvent(self, event):
#         painter = QPainter(self)
#         painter.setPen(Qt.NoPen)
#         painter.setBrush(Qt.lightGray)
#         painter.drawRoundedRect(self.rect(), 10, 10)
# PySide


# PyQt
from PyQt5.QtCore import Qt, QRectF
from PyQt5.QtWidgets import QPushButton
from PyQt5.QtGui import QPainter, QPen, QColor, QFont
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QPushButton = createWidgetMenuBase(QPushButton)

class QtBoxStyleButton11(QPushButton):
    def __init__(self):
        super(QtBoxStyleButton11, self).__init__(str(Path(__file__)))
        self.btn_state = "off"
        self.setFixedSize(90, 90)

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)

        if self.btn_state == "on":
            font_size = 15
            shadow_color = "#b0b0b0"
            white_part_color = "#eeeeee"
        else:
            font_size = 16
            shadow_color = "#d0d0d0"
            white_part_color = "#fcfcfc"

        painter.setPen(Qt.NoPen)
        painter.setBrush(QColor(shadow_color))
        btn_shadow_part_rect = self.rect()
        painter.drawRoundedRect(btn_shadow_part_rect, 5, 5)

        offset = 20
        btn_white_part_rect = QRectF(self.rect().left()+offset/2, self.rect().top()+offset/4, self.rect().width()-offset, self.rect().height()-offset)
        painter.setBrush(QColor(white_part_color))
        painter.drawRoundedRect(btn_white_part_rect, 5, 5)

        pen = QPen(Qt.SolidLine)
        pen.setColor(Qt.black)
        painter.setPen(pen)

        font = QFont()
        font.setBold(True)
        font.setPixelSize(font_size)
        painter.setFont(font)
        painter.drawText(int(btn_white_part_rect.x()+2), int(btn_white_part_rect.y()+font_size), "BTN")

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.setFixedSize(86, 86)
            self.btn_state = "on"
            self.update()

    def mouseReleaseEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.setFixedSize(90, 90)
            self.btn_state = "off"
            self.update()
# PyQt

# PySide
# from PySide2.QtCore import Qt, QRectF
# from PySide2.QtWidgets import QPushButton
# from PySide2.QtGui import QPainter, QPen, QColor, QFont


# class QtBoxStyleButton11(QPushButton):
#     def __init__(self):
#         super(QtBoxStyleButton11, self).__init__()
#         self.btn_state = "off"
#         self.setFixedSize(90, 90)

#     def paintEvent(self, event):
#         painter = QPainter(self)
#         painter.setRenderHint(QPainter.Antialiasing)

#         if self.btn_state == "on":
#             font_size = 15
#             shadow_color = "#b0b0b0"
#             white_part_color = "#eeeeee"
#         else:
#             font_size = 16
#             shadow_color = "#d0d0d0"
#             white_part_color = "#fcfcfc"

#         painter.setPen(Qt.NoPen)
#         painter.setBrush(QColor(shadow_color))
#         btn_shadow_part_rect = self.rect()
#         painter.drawRoundedRect(btn_shadow_part_rect, 5, 5)

#         offset = 20
#         btn_white_part_rect = QRectF(self.rect().left()+offset/2, self.rect().top()+offset/4, self.rect().width()-offset, self.rect().height()-offset)
#         painter.setBrush(QColor(white_part_color))
#         painter.drawRoundedRect(btn_white_part_rect, 5, 5)

#         pen = QPen(Qt.SolidLine)
#         pen.setColor(Qt.black)
#         painter.setPen(pen)

#         font = QFont()
#         font.setBold(True)
#         font.setPixelSize(font_size)
#         painter.setFont(font)
#         painter.drawText(int(btn_white_part_rect.x()+2), int(btn_white_part_rect.y()+font_size), "BTN")

#     def mousePressEvent(self, event):
#         if event.button() == Qt.LeftButton:
#             self.setFixedSize(86, 86)
#             self.btn_state = "on"
#             self.update()

#     def mouseReleaseEvent(self, event):
#         if event.button() == Qt.LeftButton:
#             self.setFixedSize(90, 90)
#             self.btn_state = "off"
#             self.update()
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QCheckBox
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QCheckBox = createWidgetMenuBase(QCheckBox)

class QtBoxFuncCheckBox1(QCheckBox):
    def __init__(self):
        super(QtBoxFuncCheckBox1, self).__init__(str(Path(__file__)))
        self.setTristate(True)
        self.setText("UnChecked")
        self.stateChanged.connect(self.update_text)

    def update_text(self):
        if self.checkState() == Qt.Unchecked:
            self.setText("UnChecked")
        elif self.checkState() == Qt.PartiallyChecked:
            self.setText("PartiallyChecked")
        elif self.checkState() == Qt.Checked:
            self.setText("Checked")
# PyQt

# PySide
# from PyQt5.QtCore import Qt
# from PyQt5.QtWidgets import QCheckBox


# class QtBoxFuncCheckBox1(QCheckBox):
#     def __init__(self):
#         super(QtBoxFuncCheckBox1, self).__init__()
#         self.setTristate(True)
#         self.setText("UnChecked")
#         self.stateChanged.connect(self.update_text)

#     def update_text(self):
#         if self.checkState() == Qt.Unchecked:
#             self.setText("UnChecked")
#         elif self.checkState() == Qt.PartiallyChecked:
#             self.setText("PartiallyChecked")
#         elif self.checkState() == Qt.Checked:
#             self.setText("Checked")
# PySide



# PyQt
from PyQt5.QtWidgets import QSlider
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QSlider = createWidgetMenuBase(QSlider)

class QtBoxStyleSlider2(QSlider):
    def __init__(self):
        super(QtBoxStyleSlider2, self).__init__(str(Path(__file__)))
        self.setStyleSheet("""
        QSlider::groove:vertical {
            background-color: #eaebed;
            border: none;
            width: 6px;
            border-radius: 3px;
        }

        QSlider::handle:vertical {
            background-color: #d33124;
            height:6px;
            margin: 0px -1px 0px -1px;
            border-radius: 3px;
        }
        
        QSlider::add-page:vertical {
            background-color: #d33124;
        }
        """)
# PyQt

# PySide
# from PySide2.QtWidgets import QSlider


# class QtBoxStyleSlider2(QSlider):
#     def __init__(self):
#         super(QtBoxStyleSlider2, self).__init__()
#         self.setStyleSheet("""
#         QSlider::groove:vertical {
#             background-color: #eaebed;
#             border: none;
#             width: 6px;
#             border-radius: 3px;
#         }

#         QSlider::handle:vertical {
#             background-color: #d33124;
#             height:6px;
#             margin: 0px -1px 0px -1px;
#             border-radius: 3px;
#         }

#         QSlider::add-page:vertical {
#             background-color: #d33124;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtGui import QMovie
from PyQt5.QtWidgets import QLabel
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLabel = createWidgetMenuBase(QLabel)
PATH_TO_GIF = str(Path(__file__).parent.parent.parent / "res/images/exercise").replace("\\", "/")

class QtBoxFuncLabel4(QLabel):
    def __init__(self):
        super(QtBoxFuncLabel4, self).__init__(str(Path(__file__)))
        movie = QMovie()
        movie.setFileName(PATH_TO_GIF)
        self.setMovie(movie)
        movie.start()
# PyQt

# PySide
# from PySide2.QtGui import QMovie
# from PySide2.QtWidgets import QLabel


# class QtBoxFuncLabel4(QLabel):
#     def __init__(self):
#         super(QtBoxFuncLabel4, self).__init__(str(Path(__file__)))
#         movie = QMovie()
#         movie.setFileName(PATH_TO_GIF)
#         self.setMovie(movie)
#         movie.start()
# PySide


# PyQt
from PyQt5.QtGui import QRegExpValidator
from PyQt5.QtWidgets import QLineEdit
from PyQt5.QtCore import QRegExp
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLineEdit = createWidgetMenuBase(QLineEdit)

class QtBoxFuncLineEdit2(QLineEdit):
    def __init__(self):
        super(QtBoxFuncLineEdit2, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 30)
        self.setPlaceholderText("Numbers only")

        reg_exp = QRegExp("[0-9]+")
        self.setValidator(QRegExpValidator(reg_exp))
# PyQt

# PySide
# from PySide2.QtGui import QRegExpValidator
# from PySide2.QtWidgets import QLineEdit
# from PySide2.QtCore import QRegExp


# class QtBoxFuncLineEdit2(QLineEdit):
#     def __init__(self):
#         super(QtBoxFuncLineEdit2, self).__init__()
#         self.setFixedSize(150, 30)
#         self.setPlaceholderText("Numbers only")

#         reg_exp = QRegExp("[0-9]+")
#         self.setValidator(QRegExpValidator(reg_exp))
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QSlider
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QSlider = createWidgetMenuBase(QSlider)

class QtBoxStyleSlider3(QSlider):
    def __init__(self):
        super(QtBoxStyleSlider3, self).__init__(str(Path(__file__)))
        self.setOrientation(Qt.Horizontal)
        self.setStyleSheet("""
        QSlider::groove:horizontal {
            border: none;
            height: 2px;
            background-color: lightgray;
        }

        QSlider::handle:horizontal {
            background-color: #f1964c;
            width: 16px;
            margin: -8px 0px -8px 0px;
            border-radius: 8px;
        }

        QSlider::sub-page:horizontal {
            background-color: #f1964c;
        }
        """)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QSlider


# class QtBoxStyleSlider3(QSlider):
#     def __init__(self):
#         super(QtBoxStyleSlider3, self).__init__()
#         self.setOrientation(Qt.Horizontal)
#         self.setStyleSheet("""
#         QSlider::groove:horizontal {
#             border: none;
#             height: 2px;
#             background-color: lightgray;
#         }

#         QSlider::handle:horizontal {
#             background-color: #f1964c;
#             width: 16px;
#             margin-top: -8px 0px -8px 0px;
#             border-radius: 8px;
#         }

#         QSlider::sub-page:horizontal {
#             background-color: #f1964c;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtCore import Qt, QRect
from PyQt5.QtGui import QPainter, QPen, QPalette, QColor
from PyQt5.QtWidgets import QWidget
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QWidget = createWidgetMenuBase(QWidget)

class QtBoxFuncWidget1(QWidget):
    def __init__(self):
        super(QtBoxFuncWidget1, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 150)
        self.rect_list = []
        self.rect_start_point = None
        self.rect_end_point = None
        self.set_background_color()

    def set_background_color(self):
        palette = QPalette()
        palette.setColor(QPalette.Background, QColor(100, 100, 100, 255))
        self.setPalette(palette)
        self.setAutoFillBackground(True)

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.rect_start_point = event.pos()
            self.rect_end_point = event.pos()
            self.update()

    def mouseMoveEvent(self, event):
        if event.buttons() == Qt.LeftButton:
            self.rect_end_point = event.pos()
            self.update()

    def mouseReleaseEvent(self, event):
        if self.rect_start_point and self.rect_end_point:
            self.rect_list.append((self.rect_start_point, self.rect_end_point))
            self.update()

        self.rect_start_point = None
        self.rect_end_point = None

    def paintEvent(self, event):
        super(QtBoxFuncWidget1, self).paintEvent(event)
        painter = QPainter(self)
        pen = QPen(Qt.SolidLine)
        pen.setColor(Qt.green)
        painter.setPen(pen)

        if self.rect_start_point and self.rect_end_point:
            painter.drawText(self.rect_start_point, f"Rect {len(self.rect_list) + 1}")
            rect = QRect(self.rect_start_point, self.rect_end_point)
            painter.drawRect(rect)

        for i, point_tuple in enumerate(self.rect_list):
            painter.drawText(point_tuple[0], f"Rect {i + 1}")
            rect = QRect(point_tuple[0], point_tuple[1])
            painter.drawRect(rect)
# PyQt

# PySide
# from PySide2.QtCore import Qt, QRect
# from PySide2.QtGui import QPainter, QPen, QPalette, QColor
# from PySide2.QtWidgets import QWidget


# class QtBoxFuncWidget1(QWidget):
#     def __init__(self):
#         super(QtBoxFuncWidget1, self).__init__()
#         self.setFixedSize(150, 150)
#         self.rect_list = []
#         self.rect_start_point = None
#         self.rect_end_point = None
#         self.set_background_color()

#     def set_background_color(self):
#         palette = QPalette()
#         palette.setColor(QPalette.Background, QColor(100, 100, 100, 255))
#         self.setPalette(palette)
#         self.setAutoFillBackground(True)

#     def mousePressEvent(self, event):
#         if event.button() == Qt.LeftButton:
#             self.rect_start_point = event.pos()
#             self.rect_end_point = event.pos()
#             self.update()

#     def mouseMoveEvent(self, event):
#         if event.buttons() == Qt.LeftButton:
#             self.rect_end_point = event.pos()
#             self.update()

#     def mouseReleaseEvent(self, event):
#         if self.rect_start_point and self.rect_end_point:
#             self.rect_list.append((self.rect_start_point, self.rect_end_point))
#             self.update()

#         self.rect_start_point = None
#         self.rect_end_point = None

#     def paintEvent(self, event):
#         super(QtBoxFuncWidget1, self).paintEvent(event)
#         painter = QPainter(self)
#         pen = QPen(Qt.SolidLine)
#         pen.setColor(Qt.green)
#         painter.setPen(pen)

#         if self.rect_start_point and self.rect_end_point:
#             painter.drawText(self.rect_start_point, f"Rect {len(self.rect_list) + 1}")
#             rect = QRect(self.rect_start_point, self.rect_end_point)
#             painter.drawRect(rect)

#         for i, point_tuple in enumerate(self.rect_list):
#             painter.drawText(point_tuple[0], f"Rect {i + 1}")
#             rect = QRect(point_tuple[0], point_tuple[1])
#             painter.drawRect(rect)
# PySide


import re
import sys
import json
import webbrowser
from pathlib import Path

from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

from qtbox.utils.check import check_update
from qtbox.utils.editor import QSSEditor
from qtbox.utils.viewer import CodeViewer
from qtbox.utils.message import MessageBox
from qtbox.utils.title import TitleWidget as WindowTitle

from qtbox.checkbox import func as func_checkbox
from qtbox.checkbox import style as style_checkbox
from qtbox.combobox import func as func_combobox
from qtbox.combobox import style as style_combobox
from qtbox.dial import func as func_dial
from qtbox.dial import style as style_dial
from qtbox.label import func as func_label
from qtbox.label import style as style_label
from qtbox.lcdnumber import func as func_lcdnumber
from qtbox.lcdnumber import style as style_lcdnumber
from qtbox.lineedit import func as func_lineedit
from qtbox.lineedit import style as style_lineedit
from qtbox.listwidget import func as func_listwidget
from qtbox.listwidget import style as style_listwidget
from qtbox.progressbar import func as func_progressbar
from qtbox.progressbar import style as style_progressbar
from qtbox.pushbutton import func as func_pushbutton
from qtbox.pushbutton import style as style_pushbutton
from qtbox.slider import func as func_slider
from qtbox.slider import style as style_slider
from qtbox.spinbox import func as func_spinbox
from qtbox.spinbox import style as style_spinbox
from qtbox.widget import func as func_widget
from qtbox.widget import style as style_widget

VERSION = "1.1.3"

UPDATES = """
1. Added 1 func demo for QLineEdit.\n
2. Added 1 func demo for QWidget.\n
3. Added a sponsor part. \n
4. Fixed a crash problem.
"""

RES_PATH = Path(__file__).parent / "res"


class WindowBody(QWidget):
    switch_theme_signal = pyqtSignal()

    def __init__(self):
        super(WindowBody, self).__init__()
        self.btn_list_widget = QListWidget()
        self.btn_list = [QPushButton(txt) for txt in ["QCheckBox", "QComboBox", "QDial",
                                                      "QLabel", "QLCDNumber", "QLineEdit",
                                                      "QListWidget", "QProgressBar", "QPushButton",
                                                      "QSlider", "QSpinBox", "QWidget"]]
        self.func_widget = QWidget()
        self.style_widget = QWidget()
        self.func_tab = QScrollArea()
        self.style_tab = QScrollArea()
        self.tab_widget = QTabWidget()

        self.editor_btn = QPushButton()
        self.sponsor_btn = QPushButton()
        self.doc_btn = QPushButton()
        self.about_btn = QPushButton()
        self.switch_btn = QPushButton()

        self.qss_editor = QSSEditor()
        self.code_viewer = CodeViewer()
        self.updates_message_box = MessageBox()
        self.sponsor_message_box = MessageBox()

        self.grid_layout_children = []
        self.grid_layout = QGridLayout()

        self.checkbox_dict = {
            "func": [func_checkbox.demo1.QtBoxFuncCheckBox1, func_checkbox.demo2.QtBoxFuncCheckBox2],
            "style": [style_checkbox.demo1.QtBoxStyleCheckBox1, style_checkbox.demo2.QtBoxStyleCheckBox2, style_checkbox.demo3.QtBoxStyleCheckBox3,
                      style_checkbox.demo4.QtBoxStyleCheckBox4]
        }

        self.combobox_dict = {
            "func": [func_combobox.demo1.QtBoxFuncComboBox1, func_combobox.demo2.QtBoxFuncComboBox2],
            "style": [style_combobox.demo1.QtBoxStyleComboBox1, style_combobox.demo2.QtBoxStyleComboBox2, style_combobox.demo3.QtBoxStyleComboBox3]
        }

        self.dial_dict = {
            "func": [func_dial.demo1.QtBoxFuncDial1],
            "style": [style_dial.demo1.QtBoxStyleDial1, style_dial.demo2.QtBoxStyleDial2]
        }

        self.label_dict = {
            "func": [func_label.demo1.QtBoxFuncLabel1, func_label.demo2.QtBoxFuncLabel2, func_label.demo3.QtBoxFuncLabel3,
                     func_label.demo4.QtBoxFuncLabel4, func_label.demo5.QtBoxFuncLabel5, func_label.demo6.QtBoxFuncLabel6,
                     func_label.demo7.QtBoxFuncLabel7],
            "style": [style_label.demo1.QtBoxStyleLabel1, style_label.demo2.QtBoxStyleLabel2, style_label.demo3.QtBoxStyleLabel3,
                      style_label.demo4.QtBoxStyleLabel4]
        }

        self.lcdnumber_dict = {
            "func": [func_lcdnumber.demo1.QtBoxFuncLCDNumber1, func_lcdnumber.demo2.QtBoxFuncLCDNumber2, func_lcdnumber.demo3.QtBoxFuncLCDNumber3],
            "style": [style_lcdnumber.demo1.QtBoxStyleLCDNumber1, style_lcdnumber.demo2.QtBoxStyleLCDNumber2, style_lcdnumber.demo3.QtBoxStyleLCDNumber3,
                      style_lcdnumber.demo4.QtBoxStyleLCDNumber4, style_lcdnumber.demo5.QtBoxStyleLCDNumber5]
        }

        self.lineedit_dict = {
            "func": [func_lineedit.demo1.QtBoxFuncLineEdit1, func_lineedit.demo2.QtBoxFuncLineEdit2, func_lineedit.demo3.QtBoxFuncLineEdit3,
                     func_lineedit.demo4.QtBoxFuncLineEdit4, func_lineedit.demo5.QtBoxFuncLineEdit5],
            "style": [style_lineedit.demo1.QtBoxStyleLineEdit1, style_lineedit.demo2.QtBoxStyleLineEdit2, style_lineedit.demo3.QtBoxStyleLineEdit3,
                      style_lineedit.demo4.QtBoxStyleLineEdit4, style_lineedit.demo5.QtBoxStyleLineEdit5, style_lineedit.demo6.QtBoxStyleLineEdit6,
                      style_lineedit.demo7.QtBoxStyleLineEdit7]
        }

        self.listwidget_dict = {
            "func": [func_listwidget.demo1.QtBoxFuncListWidget1],
            "style": [style_listwidget.demo1.QtBoxStyleListWidget1, style_listwidget.demo2.QtBoxStyleListWidget2]
        }

        self.progressbar_dict = {
            "func": [func_progressbar.demo1.QtBoxFuncProgressBar1, func_progressbar.demo2.QtBoxFuncProgressBar2],
            "style": [style_progressbar.demo1.QtBoxStyleProgressBar1, style_progressbar.demo2.QtBoxStyleProgressBar2, style_progressbar.demo3.QtBoxStyleProgressBar3,
                      style_progressbar.demo4.QtBoxStyleProgressBar4, style_progressbar.demo5.QtBoxStyleProgressBar5, style_progressbar.demo6.QtBoxStyleProgressBar6,
                      style_progressbar.demo7.QtBoxStyleProgressBar7, style_progressbar.demo8.QtBoxStyleProgressBar8, style_progressbar.demo9.QtBoxStyleProgressBar9]
        }

        self.pushbutton_dict = {
            "func": [func_pushbutton.demo1.QtBoxFuncButton1, func_pushbutton.demo2.QtBoxFuncButton2, func_pushbutton.demo3.QtBoxFuncButton3],
            "style": [style_pushbutton.demo1.QtBoxStyleButton1, style_pushbutton.demo2.QtBoxStyleButton2, style_pushbutton.demo3.QtBoxStyleButton3,
                      style_pushbutton.demo4.QtBoxStyleButton4, style_pushbutton.demo5.QtBoxStyleButton5, style_pushbutton.demo6.QtBoxStyleButton6,
                      style_pushbutton.demo7.QtBoxStyleButton7, style_pushbutton.demo8.QtBoxStyleButton8, style_pushbutton.demo9.QtBoxStyleButton9,
                      style_pushbutton.demo10.QtBoxStyleButton10, style_pushbutton.demo11.QtBoxStyleButton11, style_pushbutton.demo12.QtBoxStyleButton12]
        }

        self.slider_dict = {
            "func": [func_slider.demo1.QtBoxFuncSlider1],
            "style": [style_slider.demo1.QtBoxStyleSlider1, style_slider.demo2.QtBoxStyleSlider2, style_slider.demo3.QtBoxStyleSlider3,
                      style_slider.demo4.QtBoxStyleSlider4, style_slider.demo5.QtBoxStyleSlider5]
        }

        self.spinbox_dict = {
            "func": [func_spinbox.demo1.QtBoxFuncSpinBox1],
            "style": [style_spinbox.demo1.QtBoxStyleSpinBox1]
        }

        self.widget_dict = {
            "func": [func_widget.demo1.QtBoxFuncWidget1, func_widget.demo2.QtBoxFuncWidget2],
            "style": [style_widget.demo1.QtBoxStyleWidget1, style_widget.demo2.QtBoxStyleWidget2]
        }

        self.set_up()

    def set_up(self):
        self.set_object_name()
        self.set_style_sheet()
        self.set_widget()
        self.set_signal()
        self.set_layout()

        self.btn_list[0].setCheckable(True)
        self.btn_list[0].setChecked(True)
        self.set_tab_content("QCheckBox")

    def set_object_name(self):
        self.setObjectName("windowBody")
        self.btn_list_widget.setObjectName("btnList")

        for btn in self.btn_list:
            btn.setObjectName("listBtn")

        self.editor_btn.setObjectName("editorBtn")
        self.sponsor_btn.setObjectName("sponsorBtn")
        self.doc_btn.setObjectName("docBtn")
        self.about_btn.setObjectName("aboutBtn")
        self.switch_btn.setObjectName("switchBtn")

        self.tab_widget.setObjectName("tabWidget")
        self.func_tab.setObjectName("funcScroll")
        self.style_tab.setObjectName("styleScroll")
        self.func_widget.setObjectName("funcWidget")
        self.style_widget.setObjectName("styleWidget")

    def set_style_sheet(self):
        with open(RES_PATH / "config.json", "r", encoding="utf-8") as f:
            theme = json.loads(f.read())["theme"]

        with open(RES_PATH / f"qss/{theme}/window_body.qss", encoding="utf-8") as f:
            self.setStyleSheet(f.read())

    def set_widget(self):
        self.btn_list_widget.setMaximumWidth(300)
        self.btn_list_widget.setMinimumWidth(150)

        self.func_tab.setWidgetResizable(True)
        self.style_tab.setWidgetResizable(True)
        self.func_tab.setWidget(self.func_widget)
        self.style_tab.setWidget(self.style_widget)
        self.func_tab.setAlignment(Qt.AlignCenter)
        self.style_tab.setAlignment(Qt.AlignCenter)
        self.tab_widget.addTab(self.style_tab, "Style")
        self.tab_widget.addTab(self.func_tab, "Function")

        self.editor_btn.setFixedWidth(30)
        self.sponsor_btn.setFixedWidth(30)
        self.doc_btn.setFixedWidth(30)
        self.about_btn.setFixedWidth(30)
        self.switch_btn.setFixedWidth(30)
        self.editor_btn.setToolTip("Open QSS editor")
        self.sponsor_btn.setToolTip("Sponsor")
        self.doc_btn.setToolTip("Open documentation")
        self.about_btn.setToolTip("Show updates")
        self.switch_btn.setToolTip("Switch theme")
        self.editor_btn.setCursor(Qt.PointingHandCursor)
        self.sponsor_btn.setCursor(Qt.PointingHandCursor)
        self.doc_btn.setCursor(Qt.PointingHandCursor)
        self.about_btn.setCursor(Qt.PointingHandCursor)
        self.switch_btn.setCursor(Qt.PointingHandCursor)
        self.editor_btn.setIcon(QIcon(str(RES_PATH / "images/editor.png")))
        self.sponsor_btn.setIcon(QIcon(str(RES_PATH / "images/sponsor.png")))
        self.doc_btn.setIcon(QIcon(str(RES_PATH / "images/doc.png")))
        self.about_btn.setIcon(QIcon(str(RES_PATH / "images/about.png")))
        self.switch_btn.setIcon(QIcon(str(RES_PATH / "images/switch.png")))

        self.code_viewer.hide()

        desktop = QApplication.desktop()
        self.updates_message_box.move(desktop.width()//2-self.updates_message_box.width()//2, desktop.height()//2-self.updates_message_box.height()//2)
        self.updates_message_box.set_title(f"Qt Box Updates v{VERSION}")
        self.updates_message_box.set_plain_text(UPDATES)
        self.updates_message_box.content_browser.setLineWrapMode(QTextEdit.NoWrap)
        self.updates_message_box.hide()

        self.sponsor_message_box.setFixedSize(320, 205)
        self.sponsor_message_box.content_browser.setMinimumHeight(95)
        self.sponsor_message_box.move(desktop.width()//2-self.sponsor_message_box.width()//2, desktop.height()//2-self.sponsor_message_box.height()//2)
        self.sponsor_message_box.set_title("Customize")
        self.sponsor_message_box.set_html("Contact <a href='mailto:louasure@126.com' style='text-decoration:none'>louasure@126.com</a> to customize a widget. If you have any problems with PyQt or PySide, you may contact me, too. I will be very happy to provide solutions. <a href='https://github.com/la-vie-est-belle/qtbox' style='text-decoration:none'>Sponsors'</a> needs will be treated with top priority. (๑•̀ㅂ•́)و✧")
        self.sponsor_message_box.content_browser.setOpenExternalLinks(True)
        self.sponsor_message_box.content_browser.setAlignment(Qt.AlignJustify)
        self.sponsor_message_box.hide()

    def set_signal(self):
        for btn in self.btn_list:
            btn.clicked.connect(self.change_widget)

        self.editor_btn.clicked.connect(self.open_editor)
        self.sponsor_btn.clicked.connect(self.show_sponsor_message)
        self.doc_btn.clicked.connect(self.open_doc)
        self.about_btn.clicked.connect(self.show_updates)
        self.switch_btn.clicked.connect(self.switch_theme)
        self.tab_widget.currentChanged.connect(self.change_tab)

    def set_layout(self):
        v_layout1 = QVBoxLayout()
        h_layout1 = QHBoxLayout()
        h_layout2 = QHBoxLayout()
        v_layout1.setContentsMargins(0, 0, 0, 0)
        h_layout1.setContentsMargins(0, 0, 0, 0)

        for btn in self.btn_list:
            item = QListWidgetItem()
            item.setSizeHint(QSize(150, 50))
            self.btn_list_widget.addItem(item)
            self.btn_list_widget.setItemWidget(item, btn)

        h_layout1.addWidget(self.editor_btn)
        h_layout1.addWidget(self.sponsor_btn)
        h_layout1.addWidget(self.doc_btn)
        h_layout1.addWidget(self.about_btn)
        h_layout1.addWidget(self.switch_btn)
        h_layout1.setSpacing(30)

        v_layout1.addLayout(h_layout1)
        v_layout1.addWidget(self.btn_list_widget)
        v_layout1.setSpacing(6)
        h_layout2.addLayout(v_layout1)
        h_layout2.addWidget(self.tab_widget)
        self.setLayout(h_layout2)

        self.grid_layout.setVerticalSpacing(70)
        self.grid_layout.setHorizontalSpacing(60)

    def change_widget(self):
        self.update_btn_style()
        self.clear_grid_layout()
        self.set_tab_content(self.sender().text())

    def change_tab(self):
        """change between Style and Function tab"""
        self.clear_grid_layout()

        for btn in self.btn_list:
            if btn.isChecked():
                self.set_tab_content(btn.text())
                break

    def update_btn_style(self):
        for btn in self.btn_list:
            btn.setCheckable(False)
            btn.setChecked(False)

        self.sender().setCheckable(True)
        self.sender().setChecked(True)
        self.btn_list_widget.update()

    def clear_grid_layout(self):
        for widget in self.grid_layout_children:
            widget.deleteLater()
        self.grid_layout_children = []
        self.tab_widget.update()

    def set_tab_content(self, btn_txt):
        widget_list = []
        tab_index = self.tab_widget.currentIndex()

        key = "style" if tab_index == 0 else "func"
        if btn_txt == "QComboBox":
            widget_list = self.combobox_dict[key]

        elif btn_txt == "QCheckBox":
            widget_list = self.checkbox_dict[key]

        elif btn_txt == "QDial":
            widget_list = self.dial_dict[key]

        elif btn_txt == "QLabel":
            widget_list = self.label_dict[key]

        elif btn_txt == "QLCDNumber":
            widget_list = self.lcdnumber_dict[key]

        elif btn_txt == "QLineEdit":
            widget_list = self.lineedit_dict[key]

        elif btn_txt == "QListWidget":
            widget_list = self.listwidget_dict[key]

        elif btn_txt == "QProgressBar":
            widget_list = self.progressbar_dict[key]

        elif btn_txt == "QPushButton":
            widget_list = self.pushbutton_dict[key]

        elif btn_txt == "QSlider":
            widget_list = self.slider_dict[key]

        elif btn_txt == "QSpinBox":
            widget_list = self.spinbox_dict[key]

        elif btn_txt == "QWidget":
            widget_list = self.widget_dict[key]

        if tab_index == 0:
            self.set_style_tab_content(widget_list)
        else:
            self.set_func_tab_content(widget_list)

    def set_style_tab_content(self, widget_list):
        self.add_widget_list_to_grid_layout(widget_list)
        self.style_widget.setLayout(self.grid_layout)

    def set_func_tab_content(self, widget_list):
        self.add_widget_list_to_grid_layout(widget_list)
        self.func_widget.setLayout(self.grid_layout)

    def add_widget_list_to_grid_layout(self, widget_list):
        row, column = 0, 0
        for widget in widget_list:
            if column != 0 and column % 3 == 0:
                row += 1
                column = 0

            widget = widget()
            widget.view_code_signal.connect(self.view_code)
            widget.download_code_signal.connect(self.download_code)
            self.grid_layout.addWidget(widget, row, column, 1, 1, Qt.AlignCenter)
            self.grid_layout_children.append(widget)
            column += 1

    def view_code(self, code_file_path):
        pyqt_code, pyside_code = self.get_code_content(code_file_path)
        self.code_viewer.set_pyqt_and_pyside_property(pyqt_code, pyside_code)
        self.code_viewer.set_code(pyqt_code)
        self.code_viewer.show()
        self.code_viewer.raise_()

    def download_code(self, code_file_path):
        path = QFileDialog.getExistingDirectory(self, "Open Directory", "./", QFileDialog.ShowDirsOnly | QFileDialog.DontResolveSymlinks)
        if not path:
            return

        pyqt_code, pyside_code = self.get_code_content(code_file_path)
        with open(path+"/pyqt_code.py", "w", encoding="utf-8") as f:
            f.write(pyqt_code)
        with open(path+"/pyside_code.py", "w", encoding="utf-8") as f:
            f.write(pyside_code)

    @staticmethod
    def get_code_content(code_file_path):
        content = ""
        with open(code_file_path, "r", encoding="utf-8") as f:
            for line in f.readlines():
                if "pathlib" not in line and "Base" not in line and '"res/images' not in line:
                    content += line
            content = content.replace("str(Path(__file__))", "").replace("%s", "PATH_TO_IMG")
            content = re.sub(r" % \(.+\)", ")", content)

        try:
            pyqt_code = re.search(r"# PyQt.+# PyQt", content, re.DOTALL).group().replace("# PyQt\n", "").replace("# PyQt", "")
        except AttributeError:
            pyqt_code = ""

        try:
            pyside_code = re.search(r"# PySide.+# PySide", content, re.DOTALL).group().replace("# PySide\n", "").replace("# PySide", "").replace("# ", "")
        except AttributeError:
            pyside_code = ""

        return pyqt_code, pyside_code

    def open_editor(self):
        self.qss_editor.show()

    def show_sponsor_message(self):
        self.sponsor_message_box.show()
        self.sponsor_message_box.raise_()

    @staticmethod
    def open_doc():
        webbrowser.open("https://github.com/la-vie-est-belle/qtbox")

    def show_updates(self):
        self.updates_message_box.show()
        self.updates_message_box.raise_()

    def switch_theme(self):
        self.switch_theme_signal.emit()

    def reload_style_sheet(self):
        self.set_style_sheet()
        self.qss_editor.reload_style_sheet()
        self.code_viewer.reload_style_sheet()
        self.updates_message_box.reload_style_sheet()
        self.sponsor_message_box.reload_style_sheet()

        for widget in self.grid_layout_children:
            widget.context_menu.reload_style_sheet()


class Window(QWidget):
    def __init__(self):
        super(Window, self).__init__()
        self.window_title = WindowTitle()
        self.window_body = WindowBody()

        self.start_x = None
        self.start_y = None

        self.bg_color = None

        self.set_up()

    def set_up(self):
        self.set_window_attr()
        self.set_object_name()
        self.set_style_sheet()
        self.set_signal()
        self.set_layout()

    def set_window_attr(self):
        self.resize(1000, 600)
        self.setWindowTitle("Qt Box")
        self.setWindowFlag(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setWindowIcon(QIcon(str(RES_PATH / "images/icon.png")))

    def set_object_name(self):
        self.setObjectName("window")

    def set_style_sheet(self):
        with open(RES_PATH / "config.json", "r", encoding="utf-8") as f:
            theme = json.loads(f.read())["theme"]

        if theme == "black":
            self.bg_color = "#3b3f42"
        else:
            self.bg_color = "#f2f2f2"

    def set_signal(self):
        self.window_title.close_signal.connect(self.close)
        self.window_title.minimize_signal.connect(self.showMinimized)
        self.window_title.maximize_signal.connect(self.showMaximized)
        self.window_title.show_normal_signal.connect(self.showNormal)
        self.window_body.switch_theme_signal.connect(self.reload_style_sheet)

    def set_layout(self):
        v_layout = QVBoxLayout(self)
        v_layout.setSpacing(0)
        v_layout.addWidget(self.window_title)
        v_layout.addWidget(self.window_body)
        v_layout.setContentsMargins(0, 0, 0, 0)

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.start_x = event.x()
            self.start_y = event.y()

    def mouseMoveEvent(self, event):
        if self.start_x is None or self.start_y is None:
            return

        dis_x = event.x() - self.start_x
        dis_y = event.y() - self.start_y
        self.move(self.x() + dis_x, self.y() + dis_y)

    def mouseReleaseEvent(self, event):
        self.start_x = None
        self.start_y = None

    def paintEvent(self, event):
        super(Window, self).paintEvent(event)
        painter = QPainter(self)
        painter.setPen(Qt.NoPen)
        painter.setBrush(QColor(self.bg_color))
        painter.setRenderHint(QPainter.Antialiasing)
        painter.drawRoundedRect(self.rect(), 9, 9)

    def showMaximized(self):
        self.move(0, 0)
        desktop = QApplication.desktop()
        self.resize(desktop.width(), desktop.height())

    def showNormal(self):
        super(Window, self).showNormal()
        self.resize(1000, 600)
        desktop = QApplication.desktop()
        self.move(desktop.width() // 2 - self.width() // 2, desktop.height() // 2 - self.height() // 2)

    def reload_style_sheet(self):
        with open(RES_PATH / "config.json", "r", encoding="utf-8") as f:
            config = json.loads(f.read())

        if config["theme"] == "black":
            config["theme"] = "white"
        else:
            config["theme"] = "black"

        with open(RES_PATH / "config.json", "w", encoding="utf-8") as f:
            f.write(json.dumps(config))

        self.set_style_sheet()
        self.window_title.reload_style_sheet()
        self.window_body.reload_style_sheet()


def main():
    check_update()
    app = QApplication([])
    window = Window()
    window.show()
    sys.exit(app.exec())


if __name__ == "__main__":
    main()


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QPalette, QColor
from PyQt5.QtWidgets import QWidget, QGraphicsBlurEffect
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QWidget = createWidgetMenuBase(QWidget)

class QtBoxStyleWidget2(QWidget):
    def __init__(self):
        super(QtBoxStyleWidget2, self).__init__(str(Path(__file__)))
        self.setWindowFlag(Qt.FramelessWindowHint)
        self.setFixedSize(100, 100)
        self.set_background_color()
        self.blur()

    def set_background_color(self):
        palette = QPalette()
        palette.setColor(QPalette.Background, QColor(245, 245, 245, 220))
        self.setPalette(palette)
        self.setAutoFillBackground(True)

    def blur(self):
        blur = QGraphicsBlurEffect()
        blur.setBlurRadius(10)
        blur.setBlurHints(QGraphicsBlurEffect.QualityHint)
        self.setGraphicsEffect(blur)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtGui import QPalette, QColor
# from PySide2.QtWidgets import QWidget, QGraphicsBlurEffect


# class QtBoxStyleWidget2(QWidget):
#     def __init__(self):
#         super(QtBoxStyleWidget2, self).__init__()
#         self.setWindowFlag(Qt.FramelessWindowHint)
#         self.setFixedSize(100, 100)
#         self.set_background_color()
#         self.blur()

#     def set_background_color(self):
#         palette = QPalette()
#         palette.setColor(QPalette.Background, QColor(245, 245, 245, 220))
#         self.setPalette(palette)
#         self.setAutoFillBackground(True)

#     def blur(self):
#         blur = QGraphicsBlurEffect()
#         blur.setBlurRadius(10)
#         blur.setBlurHints(QGraphicsBlurEffect.QualityHint)
#         self.setGraphicsEffect(blur)
# PySide



# PyQt
from PyQt5.QtWidgets import QComboBox
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QComboBox = createWidgetMenuBase(QComboBox)
PATH_TO_IMG = str(Path(__file__).parent.parent.parent / "res/images/up_down.png").replace("\\", "/")

class QtBoxStyleComboBox3(QComboBox):
    def __init__(self):
        super(QtBoxStyleComboBox3, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 30)
        self.addItems(['1', '2', '3', '4', '5', '6'])
        self.setStyleSheet("""
        QComboBox {
            background-color: qlineargradient(x1:1, y1:0, x2:1, y2:1, stop:0 #f5f5f7, stop:1 #dedee0);
            border: 1px solid whitesmoke;
            border-radius: 3px;
            padding-left: 15px;
            color: gray;
        }

        QComboBox::drop-down {
            width: 22px;
            border-top-right-radius: 3px;
            border-bottom-right-radius: 3px;
        }

        QComboBox::down-arrow {
            width: 16px;
            height: 16px;
            image: url(%s);
        }

        QComboBox QAbstractItemView {
            color: gray;
            border: none;
            outline: none;
            background-color: #dedee0;
        }

        QComboBox QScrollBar:vertical {
            width: 2px;
            background-color: white;
        }

        QComboBox QScrollBar::handle:vertical {
            background-color: #b2bdaf;
        }
        """ % (PATH_TO_IMG))
# PyQt

# PySide
# from PySide2.QtWidgets import QComboBox


# class QtBoxStyleComboBox3(QComboBox):
#     def __init__(self):
#         super(QtBoxStyleComboBox3, self).__init__()
#         self.setFixedSize(150, 30)
#         self.addItems(['1', '2', '3', '4', '5', '6'])
#         self.setStyleSheet("""
#         QComboBox {
#             background-color: qlineargradient(x1:1, y1:0, x2:1, y2:1, stop:0 #f5f5f7, stop:1 #dedee0);
#             border: 1px solid whitesmoke;
#             border-radius: 3px;
#             padding-left: 15px;
#             color: gray;
#         }

#         QComboBox::drop-down {
#             width: 22px;
#             border-top-right-radius: 3px;
#             border-bottom-right-radius: 3px;
#         }

#         QComboBox::down-arrow {
#             width: 16px;
#             height: 16px;
#             image: url(%s);
#         }

#         QComboBox QAbstractItemView {
#             color: gray;
#             border: none;
#             outline: none;
#             background-color: #dedee0;
#         }

#         QComboBox QScrollBar:vertical {
#             width: 2px;
#             background-color: white;
#         }

#         QComboBox QScrollBar::handle:vertical {
#             background-color: #b2bdaf;
#         }
#         """ % (PATH_TO_IMG))
# PySide




# PyQt
from PyQt5.QtCore import QTimer, QTime
from PyQt5.QtWidgets import QLabel
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLabel = createWidgetMenuBase(QLabel)

class QtBoxFuncLabel6(QLabel):
    def __init__(self):
        super(QtBoxFuncLabel6, self).__init__(str(Path(__file__)))
        self.update_text()

        self.timer = QTimer()
        self.timer.timeout.connect(self.update_text)
        self.timer.start(1000)

    def update_text(self):
        self.setText(QTime.currentTime().toString())
# PyQt

# PySide
# from PySide2.QtCore import QTimer, QTime
# from PySide2.QtWidgets import QLabel


# class QtBoxFuncLabel6(QLabel):
#     def __init__(self):
#         super(QtBoxFuncLabel6, self).__init__(str(Path(__file__)))
#         self.update_text()

#         self.timer = QTimer()
#         self.timer.timeout.connect(self.update_text)
#         self.timer.start(1000)

#     def update_text(self):
#         self.setText(QTime.currentTime().toString())
# PySide


# PyQt
import webbrowser
from PyQt5.QtWidgets import QPushButton
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QPushButton = createWidgetMenuBase(QPushButton)

class QtBoxFuncButton3(QPushButton):
    def __init__(self):
        super(QtBoxFuncButton3, self).__init__(str(Path(__file__)))
        self.setText("Open a website")
        self.clicked.connect(self.open_a_website)

    def open_a_website(self):
        url = "https://github.com/la-vie-est-belle/qtbox"
        webbrowser.open(url)
# PyQt

# PySide
# import webbrowser
# from PySide2.QtWidgets import QPushButton


# class QtBoxFuncButton3(QPushButton):
#     def __init__(self):
#         super(QtBoxFuncButton3, self).__init__()
#         self.setText("Open a website")
#         self.clicked.connect(self.open_a_website)

#     def open_a_website(self):
#         url = "https://github.com/la-vie-est-belle/qtbox"
#         webbrowser.open(url)
# PySide


# PyQt
from PyQt5.QtGui import QPainter, QColor, QLinearGradient, QFont, QFontMetricsF, QPen
from PyQt5.QtCore import Qt, QPointF
from PyQt5.QtWidgets import QDial
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path
import math

QDial = createWidgetMenuBase(QDial)

class QtBoxStyleDial2(QDial):
    def __init__(self):
        super(QtBoxStyleDial2, self).__init__(str(Path(__file__)))
        self.setRange(100, 200)
        self.setWrapping(True)
        self.mouse_x = 0
        self.mouse_y = 0
        self.is_first_paint = True

    def mousePressEvent(self, event):
        super(QtBoxStyleDial2, self).mousePressEvent(event)
        if event.button() == Qt.LeftButton:
            self.mouse_x = event.x()
            self.mouse_y = event.y()

    def mouseMoveEvent(self, event):
        super(QtBoxStyleDial2, self).mouseMoveEvent(event)
        if event.buttons() == Qt.LeftButton:
            self.mouse_x = event.x()
            self.mouse_y = event.y()

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        pen = QPen(Qt.SolidLine)
        pen.setColor(Qt.lightGray)
        painter.setPen(pen)
        linear_gradient = QLinearGradient(self.rect().topLeft(), self.rect().bottomRight())
        linear_gradient.setColorAt(0, QColor(Qt.white))
        linear_gradient.setColorAt(1, QColor(Qt.black))
        painter.setBrush(linear_gradient)
        painter.drawEllipse(self.rect())

        if self.is_first_paint:
            self.mouse_x = self.rect().center().x()
            self.mouse_y = self.rect().bottom()
            self.is_first_paint = False

        handle_radius = 10
        painter.setBrush(Qt.white)
        handle_x, handle_y = self.get_handle_pos()
        painter.drawEllipse(QPointF(handle_x, handle_y), handle_radius, handle_radius)

        angle = self.get_angle([handle_x, handle_y])
        self.set_value(angle)

        font = QFont()
        font.setPixelSize(18)
        painter.setFont(font)
        font_metrics = QFontMetricsF(font)
        text_width = font_metrics.width(str(self.value()))
        painter.drawText(int(self.width() / 2 - text_width / 2), int(self.height() / 2), str(self.value()))

    def get_handle_pos(self):
        center_x = self.rect().center().x()
        center_y = self.rect().center().y()

        if self.mouse_x - center_x == 0 and self.mouse_y - center_y == 0:
            diff_x = center_x
            diff_y = center_y
        else:
            diff_x = self.mouse_x - center_x
            diff_y = self.mouse_y - center_y

        pos_adjust_value = 1.5
        length = math.sqrt(diff_x ** 2 + diff_y ** 2)
        ratio = length / (self.rect().width() / 2) * pos_adjust_value
        handle_x = diff_x / ratio + self.rect().center().x()
        handle_y = diff_y / ratio + self.rect().center().y()
        return handle_x, handle_y

    def get_angle(self, handle_pos):
        """get the angle between handle and the start pos"""
        center_x = self.rect().center().x()
        center_y = self.rect().center().y()
        start_pos = [self.rect().center().x(), self.rect().bottom()]

        vx1 = start_pos[0] - center_x
        vy1 = start_pos[1] - center_y
        vx2 = handle_pos[0] - center_x
        vy2 = handle_pos[1] - center_y
        angle1 = math.atan2(vy1, vx1)
        angle1 = angle1 * 180 / math.pi
        angle2 = math.atan2(vy2, vx2)
        angle2 = angle2 * 180 / math.pi

        if angle2 >= 0:
            diff_angle = angle2 - angle1
            if diff_angle < 0:
                diff_angle = 360 - abs(diff_angle)
        else:
            diff_angle = 180 + angle2 + angle1
        return diff_angle

    def set_value(self, angle):
        """set dial value via angle"""
        self.setValue(round(angle / 360 * (self.maximum() - self.minimum())) + self.minimum())
# PyQt

# PySide
# from PySide2.QtGui import QPainter, QColor, QLinearGradient, QFont, QFontMetricsF, QPen
# from PySide2.QtCore import Qt, QPointF
# from PySide2.QtWidgets import QDial
# import math


# class QtBoxStyleDial2(QDial):
#     def __init__(self):
#         super(QtBoxStyleDial2, self).__init__(str(Path(__file__)))
#         self.setRange(100, 200)
#         self.setWrapping(True)
#         self.mouse_x = 0
#         self.mouse_y = 0
#         self.is_first_paint = True

#     def mousePressEvent(self, event):
#         super(QtBoxStyleDial2, self).mousePressEvent(event)
#         if event.button() == Qt.LeftButton:
#             self.mouse_x = event.x()
#             self.mouse_y = event.y()

#     def mouseMoveEvent(self, event):
#         super(QtBoxStyleDial2, self).mouseMoveEvent(event)
#         if event.buttons() == Qt.LeftButton:
#             self.mouse_x = event.x()
#             self.mouse_y = event.y()

#     def paintEvent(self, event):
#         painter = QPainter(self)
#         painter.setRenderHint(QPainter.Antialiasing)
#         pen = QPen(Qt.SolidLine)
#         pen.setColor(Qt.lightGray)
#         painter.setPen(pen)
#         linear_gradient = QLinearGradient(self.rect().topLeft(), self.rect().bottomRight())
#         linear_gradient.setColorAt(0, QColor(Qt.white))
#         linear_gradient.setColorAt(1, QColor(Qt.black))
#         painter.setBrush(linear_gradient)
#         painter.drawEllipse(self.rect())

#         if self.is_first_paint:
#             self.mouse_x = self.rect().center().x()
#             self.mouse_y = self.rect().bottom()
#             self.is_first_paint = False

#         handle_radius = 10
#         painter.setBrush(Qt.white)
#         handle_x, handle_y = self.get_handle_pos()
#         painter.drawEllipse(QPointF(handle_x, handle_y), handle_radius, handle_radius)

#         angle = self.get_angle([handle_x, handle_y])
#         self.set_value(angle)

#         font = QFont()
#         font.setPixelSize(18)
#         painter.setFont(font)
#         font_metrics = QFontMetricsF(font)
#         text_width = font_metrics.width(str(self.value()))
#         painter.drawText(int(self.width() / 2 - text_width / 2), int(self.height() / 2), str(self.value()))

#     def get_handle_pos(self):
#         center_x = self.rect().center().x()
#         center_y = self.rect().center().y()

#         if self.mouse_x - center_x == 0 and self.mouse_y - center_y == 0:
#             diff_x = center_x
#             diff_y = center_y
#         else:
#             diff_x = self.mouse_x - center_x
#             diff_y = self.mouse_y - center_y

#         pos_adjust_value = 1.5
#         length = math.sqrt(diff_x ** 2 + diff_y ** 2)
#         ratio = length / (self.rect().width() / 2) * pos_adjust_value
#         handle_x = diff_x / ratio + self.rect().center().x()
#         handle_y = diff_y / ratio + self.rect().center().y()
#         return handle_x, handle_y

#     def get_angle(self, handle_pos):
#         """get the angle between handle and the start pos"""
#         center_x = self.rect().center().x()
#         center_y = self.rect().center().y()
#         start_pos = [self.rect().center().x(), self.rect().bottom()]

#         vx1 = start_pos[0] - center_x
#         vy1 = start_pos[1] - center_y
#         vx2 = handle_pos[0] - center_x
#         vy2 = handle_pos[1] - center_y
#         angle1 = math.atan2(vy1, vx1)
#         angle1 = angle1 * 180 / math.pi
#         angle2 = math.atan2(vy2, vx2)
#         angle2 = angle2 * 180 / math.pi

#         if angle2 >= 0:
#             diff_angle = angle2 - angle1
#             if diff_angle < 0:
#                 diff_angle = 360 - abs(diff_angle)
#         else:
#             diff_angle = 180 + angle2 + angle1
#         return diff_angle

#     def set_value(self, angle):
#         """set dial value via angle"""
#         self.setValue(round(angle / 360 * (self.maximum() - self.minimum())) + self.minimum())
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QLineEdit, QPushButton
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLineEdit = createWidgetMenuBase(QLineEdit)
PATH_TO_IMG = str(Path(__file__).parent.parent.parent / "res/images/camera.png").replace("\\", "/")

class QtBoxStyleLineEdit7(QLineEdit):
    def __init__(self):
        super(QtBoxStyleLineEdit7, self).__init__(str(Path(__file__)))
        self.setAttribute(Qt.WA_MacShowFocusRect, False)
        self.setPlaceholderText("search")
        self.setFixedSize(185, 30)
        self.setStyleSheet("""
        QLineEdit {
            border: 2px solid red;
            border-radius: 5px;
            padding-left: 5px;
            padding-right: 80px;
        }
        """)

        self.search_btn = QPushButton(self)
        self.search_btn.setText("Search")
        self.search_btn.setFixedSize(55, self.height())
        self.search_btn.setCursor(Qt.PointingHandCursor)
        self.search_btn.move(self.width() - self.search_btn.width(), 0)
        self.search_btn.setStyleSheet("""
        QPushButton {
            background-color: red;
            color: white; 
            font-size: 13px;
            border-top: 2px solid red;
            border-right: 2px solid red;
            border-bottom: 2px solid red;
            border-top-right-radius: 5px;
            border-bottom-right-radius: 5px;
        }

        QPushButton:hover {
            background-color: #f31919;
        }

        QPushButton:pressed {
            background-color: #e21818;
        }
        """)

        self.camera_btn = QPushButton(self)
        self.camera_btn.setIcon(QIcon(PATH_TO_IMG))
        self.camera_btn.setCursor(Qt.PointingHandCursor)
        self.camera_btn.setFixedSize(self.height(), self.height())
        self.camera_btn.move(self.width()-self.search_btn.width()-self.camera_btn.width(), 0)
        self.camera_btn.setStyleSheet("""
        QPushButton {
            border: none;
        }
        """)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtGui import QIcon
# from PySide2.QtWidgets import QLineEdit, QPushButton


# class QtBoxStyleLineEdit7(QLineEdit):
#     def __init__(self):
#         super(QtBoxStyleLineEdit7, self).__init__()
#         self.setAttribute(Qt.WA_MacShowFocusRect, False)
#         self.setPlaceholderText("search")
#         self.setFixedSize(185, 30)
#         self.setStyleSheet("""
#         QLineEdit {
#             border: 2px solid red;
#             border-radius: 5px;
#             padding-left: 5px;
#             padding-right: 80px;
#         }
#         """)

#         self.search_btn = QPushButton(self)
#         self.search_btn.setText("Search")
#         self.search_btn.setFixedSize(55, self.height())
#         self.search_btn.setCursor(Qt.PointingHandCursor)
#         self.search_btn.move(self.width() - self.search_btn.width(), 0)
#         self.search_btn.setStyleSheet("""
#         QPushButton {
#             background-color: red;
#             color: white;
#             font-size: 13px;
#             border-top: 2px solid red;
#             border-right: 2px solid red;
#             border-bottom: 2px solid red;
#             border-top-right-radius: 5px;
#             border-bottom-right-radius: 5px;
#         }

#         QPushButton:hover {
#             background-color: #f31919;
#         }
#
#         QPushButton:pressed {
#             background-color: #e21818;
#         }
#         """)

#         self.camera_btn = QPushButton(self)
#         self.camera_btn.setIcon(QIcon(PATH_TO_IMG))
#         self.camera_btn.setCursor(Qt.PointingHandCursor)
#         self.camera_btn.setFixedSize(self.height(), self.height())
#         self.camera_btn.move(self.width()-self.search_btn.width()-self.camera_btn.width(), 0)
#         self.camera_btn.setStyleSheet("""
#         QPushButton {
#             border: none;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtGui import QIcon
from PyQt5.QtCore import QSize, Qt
from PyQt5.QtWidgets import QPushButton
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QPushButton = createWidgetMenuBase(QPushButton)
PATH_TO_IMG = str(Path(__file__).parent.parent.parent / "res/images/search.png").replace("\\", "/")

class QtBoxStyleButton7(QPushButton):
    def __init__(self):
        super(QtBoxStyleButton7, self).__init__(str(Path(__file__)))
        self.setText("BUTTON ")
        self.setFixedSize(150, 40)
        self.setIconSize(QSize(25, 25))
        self.setIcon(QIcon(PATH_TO_IMG))
        self.setLayoutDirection(Qt.RightToLeft)
        self.setStyleSheet("""
        QPushButton {
            background-color: #292929;
            border-radius: 5px;
            font-size: 20px;
            font-weight: bold;
            color: white;
            border: 1px solid white;
        }

        QPushButton:pressed {
            background-color: black;
        }
        """)
# PyQt

# PySide
# from PySide2.QtGui import QIcon
# from PySide2.QtCore import QSize, Qt
# from PySide2.QtWidgets import QPushButton


# class QtBoxStyleButton7(QPushButton):
#     def __init__(self):
#         super(QtBoxStyleButton7, self).__init__()
#         self.setText("BUTTON ")
#         self.setFixedSize(150, 40)
#         self.setIconSize(QSize(25, 25))
#         self.setIcon(QIcon(PATH_TO_IMG))
#         self.setLayoutDirection(Qt.RightToLeft)
#         self.setStyleSheet("""
#         QPushButton {
#             background-color: #292929;
#             border-radius: 5px;
#             font-size: 20px;
#             font-weight: bold;
#             color: white;
#             border: 1px solid white;
#         }

#         QPushButton:pressed {
#             background-color: black;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtWidgets import QPushButton
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QPushButton = createWidgetMenuBase(QPushButton)

class QtBoxStyleButton6(QPushButton):
    def __init__(self):
        super(QtBoxStyleButton6, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 40)
        self.setText("BUTTON")
        self.setStyleSheet("""
        QPushButton {
            background-color: #57bd6a;
            color: #f9ffff;
            font-size: 20px;
            font-weight: bold;
            border-radius: 5px;
        }

        QPushButton:pressed {
            background-color: #4eaa5f;
        }
        """)
# PyQt

# PySide
# from PySide2.QtWidgets import QPushButton


# class QtBoxStyleButton6(QPushButton):
#     def __init__(self):
#         super(QtBoxStyleButton6, self).__init__()
#         self.setFixedSize(150, 40)
#         self.setText("BUTTON")
#         self.setStyleSheet("""
#         QPushButton {
#             background-color: #57bd6a;
#             color: #f9ffff;
#             font-size: 20px;
#             font-weight: bold;
#             border-radius: 5px;
#         }

#         QPushButton:pressed {
#             background-color: #4eaa5f;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtWidgets import QPushButton
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QPushButton = createWidgetMenuBase(QPushButton)

class QtBoxStyleButton2(QPushButton):
    def __init__(self):
        super(QtBoxStyleButton2, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 50)
        self.setText("BUTTON")
        self.setStyleSheet("""
        QPushButton {
            background-color: qlineargradient(x1:0, y1:0.5, x2:1, y2:0.5, stop:0 #47a7ed, stop: 1 #a967b2);
            color: white;
            font-size: 20px;
            font-weight: bold;
            border-radius: 25px;
        }
        
        QPushButton:hover {
            background-color: qlineargradient(x1:0, y1:0.5, x2:1, y2:0.5, stop:0 #459ee0, stop: 1 #995da1);
        }
        
        QPushButton:pressed {
               background-color: qlineargradient(x1:0, y1:0.5, x2:1, y2:0.5, stop:0 #4093d1, stop: 1 #87538e);
        }
        """)
# PyQt

# PySide
# from PySide2.QtWidgets import QPushButton


# class QtBoxStyleButton2(QPushButton):
#     def __init__(self):
#         super(QtBoxStyleButton2, self).__init__()
#         self.setFixedSize(150, 50)
#         self.setText("BUTTON")
#         self.setStyleSheet("""
#         QPushButton {
#             background-color: qlineargradient(x1:0, y1:0.5, x2:1, y2:0.5, stop:0 #47a7ed, stop: 1 #a967b2);
#             color: white;
#             font-size: 20px;
#             font-weight: bold;
#             border-radius: 25px;
#         }

#         QPushButton:hover {
#             background-color: qlineargradient(x1:0, y1:0.5, x2:1, y2:0.5, stop:0 #459ee0, stop: 1 #995da1);
#         }

#         QPushButton:pressed {
#                background-color: qlineargradient(x1:0, y1:0.5, x2:1, y2:0.5, stop:0 #4093d1, stop: 1 #87538e);
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtWidgets import QLabel
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLabel = createWidgetMenuBase(QLabel)

class QtBoxFuncLabel7(QLabel):
    def __init__(self):
        super(QtBoxFuncLabel7, self).__init__(str(Path(__file__)))
        self.setText("Drop a txt file on me.\n"
                     "I will show its content.")
        self.setAcceptDrops(True)

    def dragEnterEvent(self, event):
        super(QtBoxFuncLabel7, self).dragEnterEvent(event)
        if event.mimeData().hasUrls():
            event.acceptProposedAction()

    def dropEvent(self, event):
        super(QtBoxFuncLabel7, self).dropEvent(event)
        url = event.mimeData().urls()[0].toLocalFile()
        if not url.endswith(".txt"):
            return

        with open(url, "r", encoding="utf-8") as f:
            self.setText(f.read())
# PyQt

# PySide
# from PySide2.QtWidgets import QLabel


# class QtBoxFuncLabel7(QLabel):
#     def __init__(self):
#         super(QtBoxFuncLabel7, self).__init__()
#         self.setText("Drop a txt file on me.\n"
#                      "I will show its content.")
#         self.setAcceptDrops(True)

#     def dragEnterEvent(self, event):
#         super(QtBoxFuncLabel7, self).dragEnterEvent(event)
#         if event.mimeData().hasUrls():
#             event.acceptProposedAction()

#     def dropEvent(self, event):
#         super(QtBoxFuncLabel7, self).dropEvent(event)
#         url = event.mimeData().urls()[0].toLocalFile()
#         if not url.endswith(".txt"):
#             return

#         with open(url, "r", encoding="utf-8") as f:
#             self.setText(f.read())
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QLabel
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLabel = createWidgetMenuBase(QLabel)

class QtBoxFuncLabel5(QLabel):
    def __init__(self):
        super(QtBoxFuncLabel5, self).__init__(str(Path(__file__)))
        text = 'Lay words vertically'.replace(" ", "\n")
        self.setText(text)
        self.setAlignment(Qt.AlignCenter)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QLabel


# class QtBoxFuncLabel5(QLabel):
#     def __init__(self):
#         super(QtBoxFuncLabel5, self).__init__(str(Path(__file__)))
#         text = 'Lay words vertically'.replace(" ", "\n")
#         self.setText(text)
#         self.setAlignment(Qt.AlignCenter)
# PySide


# PyQt
from time import strftime, gmtime
from PyQt5.QtCore import QTimer
from PyQt5.QtWidgets import QLCDNumber
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLCDNumber = createWidgetMenuBase(QLCDNumber)

class QtBoxFuncLCDNumber3(QLCDNumber):
    def __init__(self):
        super(QtBoxFuncLCDNumber3, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 50)
        self.setDigitCount(8)

        self.count_down = 3600
        self.update_display()
        self.timer = QTimer()
        self.timer.timeout.connect(self.update_display)
        self.timer.start(1000)

    def update_display(self):
        self.count_down -= 1
        self.display(strftime("%H:%M:%S", gmtime(self.count_down)))
        if self.count_down <= 0:
            self.count_down = 3600
# PyQt

# PySide
# from time import strftime, gmtime
# from PySide2.QtCore import QTimer
# from PySide2.QtWidgets import QLCDNumber


# class QtBoxFuncLCDNumber3(QLCDNumber):
#     def __init__(self):
#         super(QtBoxFuncLCDNumber3, self).__init__()
#         self.setFixedSize(150, 50)
#         self.setDigitCount(8)

#         self.count_down = 3600
#         self.update_display()
#         self.timer = QTimer()
#         self.timer.timeout.connect(self.update_display)
#         self.timer.start(1000)

#     def update_display(self):
#         self.count_down -= 1
#         self.display(strftime("%H:%M:%S", gmtime(self.count_down)))
#         if self.count_down <= 0:
#             self.count_down = 3600
# PySide


# PyQt
from PyQt5.QtWidgets import QProgressBar
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QProgressBar = createWidgetMenuBase(QProgressBar)

class QtBoxStyleProgressBar6(QProgressBar):
    def __init__(self):
        super(QtBoxStyleProgressBar6, self).__init__(str(Path(__file__)))
        self.setTextVisible(False)
        self.setRange(0, 100)
        self.setValue(80)
        self.setStyleSheet("""
        QProgressBar {
            background-color: #c2c6c9;
            border: 2px solid #b0b4b7;
            border-right: 2px solid #d5d8dd;
            border-bottom: 2px solid #d5d8dd;
            border-radius: 2px;
        }

        QProgressBar::chunk {
            background-color: #0c107b;
            border-radius: 1px;
            width: 9px;
            margin: 0.5px;
        }
        """)
# PyQt

# PySide
# from PySide2.QtWidgets import QProgressBar


# class QtBoxStyleProgressBar6(QProgressBar):
#     def __init__(self):
#         super(QtBoxStyleProgressBar6, self).__init__()
#         self.setTextVisible(False)
#         self.setRange(0, 100)
#         self.setValue(80)
#         self.setStyleSheet("""
#         QProgressBar {
#             background-color: #c2c6c9;
#             border: 2px solid #b0b4b7;
#             border-right: 2px solid #d5d8dd;
#             border-bottom: 2px solid #d5d8dd;
#             border-radius: 2px;
#         }

#         QProgressBar::chunk {
#             background-color: #0c107b;
#             border-radius: 1px;
#             width: 9px;
#             margin: 0.5px;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtGui import QRegExpValidator, QValidator
from PyQt5.QtWidgets import QLineEdit, QMessageBox
from PyQt5.QtCore import QRegExp, Qt
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLineEdit = createWidgetMenuBase(QLineEdit)

class QtBoxFuncLineEdit3(QLineEdit):
    def __init__(self):
        super(QtBoxFuncLineEdit3, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 30)
        self.setPlaceholderText("example@email.com")

        reg_exp = QRegExp("[a-zA-Z0-9-_]+@([a-z0-9A-Z]+(-[a-z0-9A-Z]+)?\\.)+[a-z]{2,}$")
        self.setValidator(QRegExpValidator(reg_exp))

    def validate_email(self):
        if self.validator().validate(self.text(), 0)[0] == QValidator.Acceptable:
            QMessageBox.information(self, "Qt Box", "Correct email format")
        else:
            QMessageBox.information(self, "Qt Box", "Wrong email format")

    def keyPressEvent(self, event):
        super(QtBoxFuncLineEdit3, self).keyPressEvent(event)
        if event.key() == Qt.Key_Enter or event.key() == Qt.Key_Return:
            self.validate_email()
# PyQt

# PySide
# from PySide2.QtGui import QRegExpValidator, QValidator
# from PySide2.QtWidgets import QLineEdit, QMessageBox
# from PySide2.QtCore import QRegExp, Qt


# class QtBoxFuncLineEdit3(QLineEdit):
#     def __init__(self):
#         super(QtBoxFuncLineEdit3, self).__init__()
#         self.setFixedSize(150, 30)
#         self.setPlaceholderText("example@email.com")

#         reg_exp = QRegExp("[a-zA-Z0-9-_]+@([a-z0-9A-Z]+(-[a-z0-9A-Z]+)?\\.)+[a-z]{2,}$")
#         self.setValidator(QRegExpValidator(reg_exp))

#     def validate_email(self):
#         if self.validator().validate(self.text(), 0)[0] == QValidator.Acceptable:
#             QMessageBox.information(self, "Qt Box", "Correct email format")
#         else:
#             QMessageBox.information(self, "Qt Box", "Wrong email format")

#     def keyPressEvent(self, event):
#         super(QtBoxFuncLineEdit3, self).keyPressEvent(event)
#         if event.key() == Qt.Key_Enter or event.key() == Qt.Key_Return:
#             self.validate_email()
# PySide


# PyQt
from PyQt5.QtWidgets import QProgressBar
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QProgressBar = createWidgetMenuBase(QProgressBar)

class QtBoxFuncProgressBar1(QProgressBar):
    def __init__(self):
        super(QtBoxFuncProgressBar1, self).__init__(str(Path(__file__)))
        self.setRange(0, 0)
# PyQt

# PySide
# from PySide2.QtWidgets import QProgressBar


# class QtBoxFuncProgressBar1(QProgressBar):
#     def __init__(self):
#         super(QtBoxFuncProgressBar1, self).__init__()
#         self.setRange(0, 0)
# PySide